function [31:0] add32;
    input [31:0] left;
    input [31:0] right;
    begin
        add32 = left + right;
    end
endfunction

function [31:0] sub32;
    input [31:0] left;
    input [31:0] right;
    begin
        sub32 = left - right;
    end
endfunction

function [31:0] and32;
    input [31:0] left;
    input [31:0] right;
    begin
        and32 = left & right;
    end
endfunction

function [31:0] or32;
    input [31:0] left;
    input [31:0] right;
    begin
        or32 = left | right;
    end
endfunction

function [31:0] xor32;
    input [31:0] left;
    input [31:0] right;
    begin
        xor32 = left ^ right;
    end
endfunction

function [31:0] shr32;
    input [31:0] left;
    input [31:0] right;
    begin
        shr32 = left >> right;
    end
endfunction

function [31:0] shl32;
    input [31:0] left;
    input [31:0] right;
    begin
        shl32 = left << right;
    end
endfunction

function testBit32;
    input [31:0] value;
    input [31:0] bitNo;
    begin
        testBit32 = neq32(and32(value,shl32(1,bitNo)),0);
    end
endfunction

function [31:0] invert32;
    input [31:0] param;
    begin
        invert32 = ~param;
    end
endfunction

function eq32;
    input [31:0] left;
    input [31:0] right;
    begin
        eq32 = left == right;
    end
endfunction

function neq32;
    input [31:0] left;
    input [31:0] right;
    begin
        neq32 = left != right;
    end
endfunction

function sgnle32;
    input [31:0] left;
    input [31:0] right;
    begin
        sgnle32 = muxB(testBit32(left,31),
                (orB(notB(testBit32(right,31)),testBit32(sub32(left,right),31))),
                (andB(notB(testBit32(right,31)),testBit32(sub32(left,right),31)))
                );//$signed(left) < $signed(right);
    end
endfunction

function le32;
    input [31:0] left;
    input [31:0] right;
    begin
        le32 = left < right;
    end
endfunction

function leq32;
    input [31:0] left;
    input [31:0] right;
    begin
        leq32 = left <= right;
    end
endfunction

function ge32;
    input [31:0] left;
    input [31:0] right;
    begin
        ge32 = left > right;
    end
endfunction

function geq32;
    input [31:0] left;
    input [31:0] right;
    begin
        geq32 = left >= right;
    end
endfunction

function inRange32;
    input [31:0] value;
    input [31:0] lowerRange;
    input [31:0] upperRange;
    begin
        inRange32 = (value>=lowerRange && value<=upperRange);
    end
endfunction

function [31:0] mux32;
    input condition;
    input [31:0] trueValue;
    input [31:0] falseValue;
    begin
        mux32 = condition ? trueValue : falseValue;
    end
endfunction

function [63:0] mux64;
    input condition;
    input [63:0] trueValue;
    input [63:0] falseValue;
    begin
        mux64 = condition ? trueValue : falseValue;
    end
endfunction

function [31:0] lower32;
    input [63:0] inp;
    begin
        lower32 = inp & 64'hFFFFFFFF;
    end
endfunction

function [63:0] add64;
    input [63:0] left;
    input [63:0] right;
    begin
        add64 = left + right;
    end
endfunction

function [63:0] sub64;
    input [63:0] left;
    input [63:0] right;
    begin
        sub64 = left - right;
    end
endfunction

function [63:0] and64;
    input [63:0] left;
    input [63:0] right;
    begin
        and64 = left & right;
    end
endfunction

function [63:0] or64;
    input [63:0] left;
    input [63:0] right;
    begin
        or64 = left | right;
    end
endfunction

function [63:0] shr64;
    input [63:0] left;
    input [31:0] right;
    begin
        shr64 = left >> right;
    end
endfunction

function [63:0] shl64;
    input [63:0] left;
    input [31:0] right;
    begin
        shl64 = left << right;
    end
endfunction

function eq64;
    input [63:0] left;
    input [63:0] right;
    begin
        neq64 = left == right;
    end
endfunction

function neq64;
    input [63:0] left;
    input [63:0] right;
    begin
        neq64 = left != right;
    end
endfunction

function testBit64;
    input [63:0] value;
    input [31:0] bitNo;
    begin
        testBit64 = neq64(and64(value,shl64(1,bitNo)),0);
    end
endfunction

function [63:0] cast64;
    input [31:0] val32;
    begin
        cast64 = val32;
    end
endfunction

function muxB;
    input condition;
    input trueValue;
    input falseValue;
    begin
        muxB = condition ? trueValue : falseValue;
    end
endfunction

function orB;
    input left;
    input right;
    begin
        orB = left || right;
    end
endfunction

function andB;
    input left;
    input right;
    begin
        andB = left && right;
    end
endfunction

function notB;
    input value;
    begin
        notB = !value;
    end
endfunction

function eqB;
    input left;
    input right;
    begin
        eqB = left == right;
    end
endfunction

function neqB;
    input left;
    input right;
    begin
        neqB = left != right;
    end
endfunction
