/*@cexp:settings:1*/
module cloned_cog_alu
(
//Eingabe- und Ausgabewerte
/*@cexp:port:0*/ input [31:0] i,           //Instruktion instr[31:26]
/*@cexp:port:0*/ input [31:0] s,          //Source Register
/*@cexp:port:0*/ input [31:0] d,          //Dest Register (als Quelle)
/*@cexp:port:0*/ input [31:0] p,           //Instruction Pointer
/*@cexp:port:0*/ input run,
/*@cexp:port:0*/ input ci,                //Carry In
/*@cexp:port:0*/ input zi,                //Zero In
/*@cexp:port:0*/ input [31:0] bus_q,
/*@cexp:port:0*/ input bus_c,

/*@cexp:port:0*/ output reg wr,           //Write Enable
/*@cexp:port:0*/ output reg[31:0] r,      //Result
/*@cexp:port:0*/ output reg co,           //Carry Out
/*@cexp:port:0*/ output reg zo            //Zero Out
); 


localparam [31:0] InstructionHubByte     = 6'b000000;
localparam [31:0] InstructionHubWord     = 6'b000001;
localparam [31:0] InstructionHubLong     = 6'b000010;
localparam [31:0] InstructionSysOp       = 6'b000011;
localparam [31:0] InstructionIllgMul     = 6'b000100;
localparam [31:0] InstructionIllgMuls    = 6'b000101;
localparam [31:0] InstructionIllgEnc     = 6'b000110;
localparam [31:0] InstructionIllgOnes    = 6'b000111;
localparam [31:0] InstructionRor        = 6'b001000;
localparam [31:0] InstructionRol        = 6'b001001;
localparam [31:0] InstructionShr        = 6'b001010;
localparam [31:0] InstructionShl        = 6'b001011;
localparam [31:0] InstructionRcr        = 6'b001100;
localparam [31:0] InstructionRcl        = 6'b001101;
localparam [31:0] InstructionSar        = 6'b001110;
localparam [31:0] InstructionRev        = 6'b001111;
localparam [31:0] InstructionMinS       = 6'b010000;
localparam [31:0] InstructionMaxS       = 6'b010001;
localparam [31:0] InstructionMin        = 6'b010010;
localparam [31:0] InstructionMax        = 6'b010011;
localparam [31:0] InstructionMovS       = 6'b010100;
localparam [31:0] InstructionMovD       = 6'b010101;
localparam [31:0] InstructionMovI       = 6'b010110;
localparam [31:0] InstructionJmpRet     = 6'b010111;
localparam [31:0] InstructionAnd        = 6'b011000;
localparam [31:0] InstructionAndN       = 6'b011001;
localparam [31:0] InstructionOr         = 6'b011010;
localparam [31:0] InstructionXor        = 6'b011011;
localparam [31:0] InstructionMuxC       = 6'b011100;
localparam [31:0] InstructionMuxNC      = 6'b011101;
localparam [31:0] InstructionMuxZ       = 6'b011110;
localparam [31:0] InstructionMuxNZ      = 6'b011111;
localparam [31:0] InstructionAdd        = 6'b100000;
localparam [31:0] InstructionSub        = 6'b100001;
localparam [31:0] InstructionAddAbs     = 6'b100010;
localparam [31:0] InstructionSubAbs     = 6'b100011;
localparam [31:0] InstructionSumC       = 6'b100100;
localparam [31:0] InstructionSumNC      = 6'b100101; 
localparam [31:0] InstructionSumZ       = 6'b100110; 
localparam [31:0] InstructionSumNZ      = 6'b100111; 
localparam [31:0] InstructionMov        = 6'b101000;
localparam [31:0] InstructionNeg        = 6'b101001;
localparam [31:0] InstructionAbs        = 6'b101010;
localparam [31:0] InstructionAbsNeg     = 6'b101011;
localparam [31:0] InstructionNegC       = 6'b101100;
localparam [31:0] InstructionNegNC      = 6'b101101;
localparam [31:0] InstructionNegZ       = 6'b101110;
localparam [31:0] InstructionNegNZ      = 6'b101111;
localparam [31:0] InstructionCmpS       = 6'b110000;
localparam [31:0] InstructionCmpSX      = 6'b110001;
localparam [31:0] InstructionAddX       = 6'b110010;
localparam [31:0] InstructionSubX       = 6'b110011;
localparam [31:0] InstructionAddS       = 6'b110100;
localparam [31:0] InstructionSubS       = 6'b110101;
localparam [31:0] InstructionAddSX      = 6'b110110;
localparam [31:0] InstructionSubSX      = 6'b110111;
localparam [31:0] InstructionCmpSub     = 6'b111000;
localparam [31:0] InstructionDJNZ       = 6'b111001;
localparam [31:0] InstructionTJNZ       = 6'b111010;
localparam [31:0] InstructionTJZ        = 6'b111011;
localparam [31:0] InstructionWaitPEq    = 6'b111100;
localparam [31:0] InstructionWaitPNe    = 6'b111101;
localparam [31:0] InstructionWaitCnt    = 6'b111110;
localparam [31:0] InstructionWaitVid    = 6'b111111;

//Lokale Variablen
/*@cexp:reg-local:0*/ reg[63:0] tmpResSigned;
/*@cexp:reg-local:0*/ reg[63:0] tmpResUnsigned;
/*@cexp:reg-local:0*/ reg[31:0] tmpResBitOp;

function getParity;
    input [31:0] value; 
    reg [31:0] tmppar;
    begin
        tmppar = xor32(value, shr32(value,16));
        tmppar = xor32(tmppar,shr32(tmppar,8));
        tmppar = xor32(tmppar,shr32(tmppar,4));
        tmppar = xor32(tmppar,shr32(tmppar,2));
        tmppar = xor32(tmppar,shr32(tmppar,1));
        getParity = testBit32(tmppar,0);
    end
endfunction

function [63:0] extendToU64ShiftSignToUpper;
    input [31:0] value; 
    begin
        extendToU64ShiftSignToUpper = or64(shl64(cast64(and32(value,32'h80000000)),1),cast64(and32(value,32'h7FFFFFFF)));
    end
endfunction

function [63:0] signedAddOrSub;
    input [31:0] dV;
    input [31:0] sV;
    input doSubtraction;
    input addExtraCarry;
    begin
        signedAddOrSub = add64(
            add64(extendToU64ShiftSignToUpper(dV),extendToU64ShiftSignToUpper(mux32(doSubtraction,invert32(sV),sV))),
            cast64(mux32(eqB(doSubtraction,addExtraCarry),32'h80000000,32'h80000001))
            );
    end
endfunction

function [31:0] getSignedResult;
    input [63:0] value;
    begin
        getSignedResult = or32(and32(lower32(shr64(value,1)),64'h80000000),and32(lower32(value),64'h7FFFFFFF));
    end
endfunction

function getSignedCarry;
    input [63:0] value;
    begin
        getSignedCarry = eqB(testBit64(value,33),testBit64(value,31));
    end
endfunction

function getSignedZero;
    input [63:0] value;
    begin
        getSignedZero = eq32(getSignedResult(value),0);
    end
endfunction

function [63:0] unsignedAddOrSub;
    input [31:0] dV;
    input [31:0] sV;
    input doSubtraction;
    input addExtraCarry;
    begin
        unsignedAddOrSub = mux64(doSubtraction,
                sub64(cast64(dV),add64(cast64(sV),cast64(mux32(addExtraCarry,1,0)))),
                add64(cast64(dV),add64(cast64(sV),cast64(mux32(addExtraCarry,1,0))))
        );
    end
endfunction

function [31:0] getUnsignedResult;
    input [63:0] value;
    begin
        getUnsignedResult = lower32(value);
    end
endfunction

function getUnsignedCarry;
    input [63:0] value;
    begin
        getUnsignedCarry = testBit64(value,32);
    end
endfunction

function getUnsignedZero;
    input [63:0] value;
    begin
        getUnsignedZero = eq32(getUnsignedResult(value),0);
    end
endfunction

function [31:0] andWithZFlag;
    input [31:0] sV;
    input [31:0] dV;
    begin
        andWithZFlag = and32(sV,dV);
    end
endfunction

function [31:0] orWithZFlag;
    input [31:0] sV;
    input [31:0] dV;
    begin
        orWithZFlag = or32(sV,dV);
    end
endfunction

function [31:0] xorWithZFlag;
    input [31:0] sV;
    input [31:0] dV;
    begin
        xorWithZFlag = xor32(sV,dV);
    end
endfunction

function [31:0] shlWithZFlag;
    input [31:0] value;
    input [31:0] shiftBy;
    begin
        shlWithZFlag = shl32(value,shiftBy);
    end
endfunction

function [31:0] shrWithZFlag;
    input [31:0] value;
    input [31:0] shiftBy;
    begin
        shrWithZFlag = shr32(value,shiftBy);
    end
endfunction

function [31:0] negWithZFlag;
    input [31:0] value;
    begin
        negWithZFlag = sub32(0,value);
    end
endfunction

function [31:0] getBitOpResult;
    input [31:0] value;
    begin
        getBitOpResult = value;
    end
endfunction

function getBitOpZero;
    input [31:0] value;
    begin
        getBitOpZero = eq32(value,0);
    end
endfunction


always @(*) begin
    /*@cexp:dummyinit-start*/
    wr=0;
    r=0;
    co=0;
    zo=0;
    r=0;
    tmpResSigned=0;
    tmpResUnsigned=0;
    tmpResBitOp=0;
    /*@cexp:dummyinit-stop*/
    
    
    //Hier geht es los
    /*@cexp:code-start:*/
    wr=1;
    zo=0;
    co=0 /*@cexp:set-valid-value*/;
    r=0 /*@cexp:set-valid-value*/;
    tmpResSigned=0;
    tmpResUnsigned=0;
    tmpResBitOp=0;
    
    //assume(i==InstructionHubByte || i==InstructionHubWord || i==InstructionHubLong || i==InstructionSysOp || i==InstructionIllgMul || i==InstructionIllgMuls || i==InstructionIllgEnc||i==InstructionIllgOnes);
    
    case (i)
        InstructionHubByte,InstructionHubWord,InstructionHubLong,InstructionSysOp,InstructionIllgMul,InstructionIllgMuls,InstructionIllgEnc,InstructionIllgOnes: begin
            co = bus_c;
            if (orB(run,neq32(and32(p,32'h1F0),32'h1F0))) begin
                r = bus_q;
                zo = eq32(r,0);
            end else begin
                r = 0;
                zo = 1;
            end
        end
        InstructionRor: begin
            co = testBit32(d,0);
            tmpResBitOp = orWithZFlag(shr32(d,and32(s,31)),shl32(d,sub32(32,and32(s,31))));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionRol: begin
            co = testBit32(d,31);
            tmpResBitOp = orWithZFlag(shl32(d,and32(s,31)),shr32(d,sub32(32,and32(s,31))));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionShr: begin
            co = testBit32(d,0);
            tmpResBitOp = shrWithZFlag(d,and32(s,31));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionShl:begin
            co = testBit32(d,31);
            tmpResBitOp = shlWithZFlag(d,and32(s,31));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionRcr:begin
            co = testBit32(d,0);
            if (ci) begin
                tmpResBitOp = orWithZFlag(shr32(d,and32(s,31)),shl32(32'hFFFFFFFF,sub32(32,and32(s,31))));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = shrWithZFlag(d,and32(s,31));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
        end
        InstructionRcl:begin
            co = testBit32(d,31);
            if (ci) begin
                tmpResBitOp = orWithZFlag(shl32(d,and32(s,31)),shr32(32'hFFFFFFFF,sub32(32,and32(s,31))));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = shlWithZFlag(d,and32(s,31));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
        end
        InstructionSar: begin
            co = testBit32(d,0);
            if (testBit32(d,31)) begin
                tmpResBitOp = orWithZFlag(shr32(d,and32(s,31)),shl32(32'hFFFFFFFF,sub32(32,and32(s,31))));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = shrWithZFlag(d,and32(s,31));
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
        end
        InstructionRev:begin
            r = or32(shr32(and32(d,32'haaaaaaaa),1),shl32(and32(d,32'h55555555),1));
            r = or32(shr32(and32(r,32'hcccccccc),2),shl32(and32(r,32'h33333333),2));
            r = or32(shr32(and32(r,32'hf0f0f0f0),4),shl32(and32(r,32'h0f0f0f0f),4));
            r = or32(shr32(and32(r,32'hff00ff00),8),shl32(and32(r,32'h00ff00ff),8));
            r = or32(shr32(r,16),shl32(r,16));
            r = shr32(r,and32(s,31));
            zo = eq32(r,0);
            co = eq32(and32(d,1),1);
        end
        InstructionMinS:begin
            zo = eq32(s,0);
            r = s;
            co = sgnle32(d,s);
            wr = co;
        end
        InstructionMaxS:begin
            zo = eq32(s,0);
            r = s;
            co = sgnle32(d,s);
            wr = notB(co);
        end
        InstructionMin:begin
            zo = eq32(s,0);
            r = s;
            tmpResUnsigned = unsignedAddOrSub(d,s,1,0);
            co = getUnsignedCarry(tmpResUnsigned);
            wr = co;
        end
        InstructionMax:begin
            zo = eq32(s,0);
            r = s;
            tmpResUnsigned = unsignedAddOrSub(d,s,1,0);
            co = getUnsignedCarry(tmpResUnsigned);
            wr = notB(co);
        end
        InstructionMovS:begin
            co = le32(d,s); //undokumentierter seiteneffekt
            tmpResBitOp = orWithZFlag(and32(d,32'hFFFFFE00),and32(s,32'h1FF));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionMovD:begin
            co = le32(d,s); //undokumentierter seiteneffekt
            tmpResBitOp = orWithZFlag(and32(d,32'hFFFC01FF),shl32(and32(s,32'h1FF),9));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionMovI:begin
            co = le32(d,s); //undokumentierter seiteneffekt
            tmpResBitOp = orWithZFlag(and32(d,32'h007FFFFF),shl32(and32(s,32'h1FF),23));
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionJmpRet:begin
            co = le32(d,s); //undokumentierter seiteneffekt
            tmpResBitOp = orWithZFlag(and32(d,32'hFFFFFE00),p);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionAnd: begin
            tmpResBitOp = andWithZFlag(s,d);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
            co = getParity(r);
        end
        InstructionAndN: begin
            tmpResBitOp = andWithZFlag(invert32(s),d);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
            co = getParity(r);
        end
        InstructionOr: begin
            tmpResBitOp = orWithZFlag(s,d);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
            co = getParity(r);
        end
        InstructionXor: begin
            tmpResBitOp = xorWithZFlag(s,d);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
            co = getParity(r);
        end
        InstructionMuxC: begin
            if (ci) begin
                tmpResBitOp = orWithZFlag(s,d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = andWithZFlag(invert32(s),d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
            co = getParity(r);
        end
        InstructionMuxNC: begin
            if (notB(ci)) begin
                tmpResBitOp = orWithZFlag(s,d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = andWithZFlag(invert32(s),d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
            co = getParity(r);
        end
        InstructionMuxZ: begin
            if (zi) begin
                tmpResBitOp = orWithZFlag(s,d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = andWithZFlag(invert32(s),d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
            co = getParity(r);
        end
        InstructionMuxNZ: begin
            if (notB(zi)) begin
                tmpResBitOp = orWithZFlag(s,d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                tmpResBitOp = andWithZFlag(invert32(s),d);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end
            co = getParity(r);
        end
        InstructionAdd: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,0);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionSub: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,1,0);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionAddAbs: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,testBit32(s,31),0);
            co = getUnsignedCarry(tmpResUnsigned);
            r = getUnsignedResult(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionSubAbs: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,notB(testBit32(s,31)),0);
            co = getUnsignedCarry(tmpResUnsigned);
            r = getUnsignedResult(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionSumC: begin
            tmpResSigned = signedAddOrSub(d,s,ci,0);
            r = getSignedResult(tmpResSigned);
            co = getSignedCarry(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionSumNC: begin
            tmpResSigned = signedAddOrSub(d,s,notB(ci),0);
            co = getSignedCarry(tmpResSigned);
            r = getSignedResult(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionSumZ: begin
            tmpResSigned = signedAddOrSub(d,s,zi,0);
            co = getSignedCarry(tmpResSigned);
            r = getSignedResult(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionSumNZ: begin
            tmpResSigned = signedAddOrSub(d,s,notB(zi),0);
            co = getSignedCarry(tmpResSigned);
            r = getSignedResult(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionMov: begin
            co = testBit32(s,31);
            zo = eq32(s,0);
            r = s;
        end
        InstructionNeg: begin
            co = testBit32(s,31);
            tmpResBitOp=negWithZFlag(s);
            r = getBitOpResult(tmpResBitOp);
            zo = getBitOpZero(tmpResBitOp);
        end
        InstructionAbs: begin
            co = testBit32(s,31);
            if (co) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionAbsNeg: begin
            co = testBit32(s,31);
            if (notB(co)) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionNegC: begin
            co = testBit32(s,31);
            if (ci) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionNegNC: begin
            co = testBit32(s,31);
            if (notB(ci)) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionNegZ: begin
            co = testBit32(s,31);
            if (zi) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionNegNZ: begin
            co = testBit32(s,31);
            if (notB(zi)) begin
                tmpResBitOp=negWithZFlag(s);
                r = getBitOpResult(tmpResBitOp);
                zo = getBitOpZero(tmpResBitOp);
            end else begin
                r = s;
                zo = eq32(s,0);
            end
        end
        InstructionCmpS: begin
            tmpResSigned = signedAddOrSub(d,s,1,0); //nötig damit das r richtig ist (undokumentiert)
            r = getSignedResult(tmpResSigned);
            co = neqB(eq32(and32(shr32(r,31),1),1),getSignedCarry(tmpResSigned));
            zo = getSignedZero(tmpResSigned);
        end
        InstructionCmpSX: begin
            tmpResSigned = signedAddOrSub(d,s,1,ci); //nötig damit das r richtig ist (undokumentiert)
            r = getSignedResult(tmpResSigned);
            co = neqB(eq32(and32(shr32(r,31),1),1),getSignedCarry(tmpResSigned));
            zo = andB(zi,getSignedZero(tmpResSigned));
        end
        InstructionAddX: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,ci);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = andB(zi,getUnsignedZero(tmpResUnsigned));
        end
        InstructionSubX: begin
            tmpResUnsigned = unsignedAddOrSub(d,s,1,ci);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = andB(zi,getUnsignedZero(tmpResUnsigned));
        end
        InstructionAddS: begin
            tmpResSigned = signedAddOrSub(d,s,0,0);
            r = getSignedResult(tmpResSigned);
            co = getSignedCarry(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionSubS: begin
            tmpResSigned = signedAddOrSub(d,s,1,0);
            r = getSignedResult(tmpResSigned);
            co = getSignedCarry(tmpResSigned);
            zo = getSignedZero(tmpResSigned);
        end
        InstructionAddSX:begin
            tmpResSigned = signedAddOrSub(d,s,0,ci);
            r = getSignedResult(tmpResSigned);
            co = getSignedCarry(tmpResSigned);
            zo = andB(zi,getSignedZero(tmpResSigned));
        end
        InstructionSubSX:begin
            tmpResSigned = signedAddOrSub(d,s,1,ci);
            r = getSignedResult(tmpResSigned);
            co = getSignedCarry(tmpResSigned);
            zo = andB(zi,getSignedZero(tmpResSigned));
        end
        InstructionCmpSub:begin
            tmpResUnsigned = unsignedAddOrSub(d,s,1,0);
            r = getUnsignedResult(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
            co = notB(getUnsignedCarry(tmpResUnsigned));
            wr = co;
        end
        InstructionDJNZ:begin
            co = eq32(d,0);
            tmpResUnsigned = unsignedAddOrSub(d,1,1,0);
            r = getUnsignedResult(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionTJNZ:begin
            zo = eq32(d,0);
            r = d;
            co = 0;
        end
        InstructionTJZ:begin
            zo = eq32(d,0);
            r = d;
            co = 0;
        end
        InstructionWaitPEq:begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,0);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionWaitPNe:begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,1);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionWaitCnt:begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,0);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
        InstructionWaitVid:begin
            tmpResUnsigned = unsignedAddOrSub(d,s,0,0);
            r = getUnsignedResult(tmpResUnsigned);
            co = getUnsignedCarry(tmpResUnsigned);
            zo = getUnsignedZero(tmpResUnsigned);
        end
    endcase
    /*@cexp:code-stop*/
    /*@cexp:always-block-end*/
end

endmodule
