module verifyequal_cog_alu (
input run,
input [31:0] d,
input ci,
input [5:0] i,
input [31:0] bus_q,
input zi,
input [8:0] p,
input [31:0] s,
input bus_c
);
wire wr_expected;
wire wr_result;
wire co_expected;
wire co_result;
wire [31:0] r_expected;
wire [31:0] r_result;
wire zo_expected;
wire zo_result;
cog_alu golden(
    .run (run),
    .wr (wr_expected),
    .d (d),
    .ci (ci),
    .i (i),
    .bus_q (bus_q),
    .co (co_expected),
    .zi (zi),
    .p (p),
    .r (r_expected),
    .s (s),
    .zo (zo_expected),
    .bus_c (bus_c));
cloned_cog_alu cloned(
    .run (run),
    .wr (wr_result),
    .d (d),
    .ci (ci),
    .i (i),
    .bus_q (bus_q),
    .co (co_result),
    .zi (zi),
    .p (p),
    .r (r_result),
    .s (s),
    .zo (zo_result),
    .bus_c (bus_c));
assert property(wr_result == wr_expected);
assert property(co_result == co_expected);
assert property(r_result == r_expected);
assert property(zo_result == zo_expected);
endmodule

//-show wr_result -show wr_expected -show co_result -show co_expected -show r_result -show r_expected -show zo_result -show zo_expected 

module cog_alu (
input run,
output wr,
input [31:0] d,
input ci,
input [5:0] i,
input [31:0] bus_q,
output co,
input zi,
input [8:0] p,
output [31:0] r,
input [31:0] s,
output zo,
input bus_c
);
wire add_c;
wire rot_c;
wire [31:0] add_d;
wire log_c;
wire [31:0] dr;
wire [31:0] rotr;
wire [31:0] add_s;
wire [31:0] add_r;
wire [31:0] log_r;
wire [62:0] rot;
wire [31:0] log_x[4];
wire [1:0] log_s;
wire [34:0] add_x;
wire add_sub;
wire [31:0] rot_r;
wire [3:0] ads;
wire add_ci;
wire [30:0] ri[8];
wire add_cs;
wire [31:0] mov_x[4];
wire add_co;
wire add_cm;
assign dr = {d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7], d[8], d[9], d[10], d[11], d[12], d[13], d[14], d[15], d[16], d[17], d[18], d[19], d[20], d[21], d[22], d[23], d[24], d[25], d[26], d[27], d[28], d[29], d[30], d[31]};
assign ri[7] = 31'h0;
assign ri[6] = {{31{d[31]}}};
assign ri[5] = {{31{ci}}};
assign ri[4] = {{31{ci}}};
assign ri[3] = 31'h0;
assign ri[2] = 31'h0;
assign ri[1] = dr[30:0];
assign ri[0] = d[30:0];
assign rot = ({ri[i[2:0]], ((i[0]) ? (dr) : (d))} >> s[4:0]);
assign rotr = {rot[0], rot[1], rot[2], rot[3], rot[4], rot[5], rot[6], rot[7], rot[8], rot[9], rot[10], rot[11], rot[12], rot[13], rot[14], rot[15], rot[16], rot[17], rot[18], rot[19], rot[20], rot[21], rot[22], rot[23], rot[24], rot[25], rot[26], rot[27], rot[28], rot[29], rot[30], rot[31]};
assign rot_r = ((((~&i[2:1]) && i[0])) ? (rotr) : (rot[31:0]));
assign rot_c = ((((~&i[2:1]) && i[0])) ? (dr[0]) : (d[0]));
assign log_s = ((i[2]) ? ({(((i[1]) ? (zi) : (ci)) ^ i[0]), 1'b0}) : ({i[1], (~^i[1:0])}));
assign log_x[3] = (d ^ s);
assign log_x[2] = (d | s);
assign log_x[1] = (d & s);
assign log_x[0] = (d & (~s));
assign mov_x[3] = {d[31:9], p};
assign mov_x[2] = {s[8:0], d[22:0]};
assign mov_x[1] = {d[31:18], s[8:0], d[8:0]};
assign mov_x[0] = {d[31:9], s[8:0]};
assign log_r = ((i[3]) ? (log_x[log_s]) : (((i[2]) ? (mov_x[i[1:0]]) : (s))));
assign log_c = (^log_r);
assign ads = {zi, ci, s[31], 1'b0};
assign add_sub = (((i[5:4] == 2'b10)) ? ((ads[i[2:1]] ^ i[0])) : (((((i[5:0] == 6'b110010) || ((i[5:0] == 6'b110100) || ((i[5:0] == 6'b110110) || (i[5:2] == 4'b1111))))) ? (1'b0) : (1'b1))));
assign add_ci = (((i[5:3] == 3'b110) && (((i[2:0] == 3'b1) || i[1]) && ci)) || ((i[4:3] == 2'b11) && (i[1:0] == 2'b1)));
assign add_d = (((i[4:3] == 2'b1)) ? (32'h0) : (d));
assign add_s = ((((i[4:0] == 5'b11001) || (i[4:1] == 4'b1101))) ? (32'hffffffff) : (((add_sub) ? ((~s)) : (s))));
assign add_x = ({1'b0, add_d[31], 1'b1, add_d[30:0], 1'b1} + {1'b0, add_s[31], 1'b0, add_s[30:0], (add_ci ^ add_sub)});
assign add_r = {add_x[33], add_x[31:1]};
assign add_co = add_x[34];
assign add_cm = (!add_x[32]);
assign add_cs = (add_co ^ (add_d[31] ^ add_s[31]));
assign add_c = (((i[5:0] == 6'b111000)) ? (add_co) : ((((i[5:3] == 3'b101)) ? (s[31]) : ((((i[5] && (i[3:2] == 2'b1))) ? ((add_co ^ add_cm)) : ((((i[4:1] == 4'b1000)) ? (add_cs) : ((add_co ^ add_sub)))))))));
assign wr = (((i[5:2] == 4'b100)) ? ((i[0] ^ ((i[1]) ? ((!add_co)) : (add_cs)))) : ((((i[5:0] == 6'b111000)) ? (add_co) : (1'b1))));
assign r = ((i[5]) ? (add_r) : (((i[4]) ? (log_r) : (((i[3]) ? (rot_r) : ((((run || (~&p[8:4]))) ? (bus_q) : (32'h0))))))));
assign co = (((i[5:3] == 3'b0)) ? (bus_c) : ((((i[5:3] == 3'b1)) ? (rot_c) : ((((i[5:3] == 3'b11)) ? (log_c) : (add_c))))));
assign zo = ((~|r) && (zi || (!((i[5:3] == 3'b110) && ((i[2:0] == 3'b1) || i[1])))));
endmodule

