// cog_ctr

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

/*@cexp:settings:2*/
module cloned_cog_ctr
(
//Eingabe- und Ausgabewerte
/*@cexp:port-ignore*/ input clk_cog,
/*@cexp:port-ignore*/ input clk_pll,
/*@cexp:port:1*/ input ena,
/*@cexp:port:1*/ input setctr,
/*@cexp:port:1*/ input setfrq,
/*@cexp:port:1*/ input setphs,
/*@cexp:port:1*/ input [31:0] data,
/*@cexp:port:1*/ input [31:0] pin_in,
/*@cexp:port:1*/ output reg [63:0] phs,
/*@cexp:port:1*/ output reg [31:0] pin_out,
/*@cexp:port:0*/ output reg pll,

//Register nach außen, nur zur Verifikation
/*@cexp:port-state*/ output reg [31:0] reg_ctr_d,
/*@cexp:port-state*/ output reg [31:0] reg_frq_d,
/*@cexp:port-state*/ output reg [31:0] reg_dly_d,
/*@cexp:port-state*/ output reg [63:0] reg_pll_fake_d,
/*@cexp:port-state*/ output reg [63:0] reg_phs_d,
/*@cexp:port-state*/ input [31:0] reg_ctr_q,
/*@cexp:port-state*/ input [31:0] reg_frq_q,
/*@cexp:port-state*/ input [1:0] reg_dly_q,
/*@cexp:port-state*/ input [35:0] reg_pll_fake_q,
/*@cexp:port-state*/ input [32:0] reg_phs_q,

//Taktflanken Signale
/*@cexp:port-clock:1*/ input posedge_clk_cog,
/*@cexp:port-clock:1*/ input posedge_clk_pll,
/*@cexp:port-clock:1*/ input negedge_ena
);

//Diese Register werden später nicht gebraucht
/*@cexp:reg-dummy*/ reg [31:0] ctrdummy1;
/*@cexp:reg-dummy*/ reg [31:0] ctrdummy2;
/*@cexp:reg-dummy*/ reg ctrdummy3;

//Interne Register
/*@cexp:reg-state:1:1*/ reg [31:0] frequency;
/*@cexp:reg-state:1:1*/ reg [63:0] phase;
/*@cexp:reg-state:0,1:1*/ reg [63:0] pllcounter;
/*@cexp:reg-state:1:1*/ reg [31:0] ctrmode;
/*@cexp:reg-state:0,1:1*/ reg [31:0] ctrplldiv;
/*@cexp:reg-state:1:1*/ reg [31:0] ctrbpin;
/*@cexp:reg-state:1:1*/ reg [31:0] ctrapin;
/*@cexp:reg-state:1:1*/ reg delayed0;
/*@cexp:reg-state:1:1*/ reg delayed1;
/*@cexp:reg-state:0,1:0*/ reg pll_saved;

always @(*) begin
    /*@cexp:dummyinit-start*/
    phs = 0;
    pin_out = 0;
    pll = 0;
    reg_ctr_d = 0;
    reg_frq_d = 0;
    reg_dly_d = 0;
    reg_pll_fake_d = 0;
    reg_phs_d = 0;
    ctrdummy1 = 0;
    ctrdummy2 = 0;
    ctrdummy3 = 0;
    frequency = 0;
    phase = 0;
    pllcounter = 0;
    ctrmode = 0;
    ctrplldiv = 0;
    ctrbpin = 0;
    ctrapin = 0;
    delayed0 = 0;
    delayed1 = 0;
    pll_saved = 0;
    /*@cexp:dummyinit-stop*/


    frequency = reg_frq_q;
    phase = reg_phs_q;
    pllcounter = reg_pll_fake_q;
    ctrdummy3 = reg_ctr_q[31];
    ctrmode = reg_ctr_q[30:26];
    ctrplldiv = reg_ctr_q[25:23];
    ctrdummy2 = reg_ctr_q[22:14];
    ctrbpin = reg_ctr_q[13:9]; //laut handbuch 14:9
    ctrdummy1 = reg_ctr_q[8:5];
    ctrapin = reg_ctr_q[4:0]; //laut handbuch 5:0
    delayed0 = reg_dly_q[0];
    delayed1 = reg_dly_q[1];
    
    //Hier geht es los
    /*@cexp:code-start:*/
    pll_saved = eq32(and32(
                    lower32(shr64(pllcounter,
                            add32(and32(invert32(ctrplldiv),7),28))
                        ),1),1);
    pll = pll_saved;
    /*@cexp:code-stop*/
    
    /*@cexp:code-start:clocks_cog*/
    phs = phase;
    pin_out = 0 /*@cexp:set-valid-value*/;
    
    case(ctrmode)
        0: begin
        end
        1: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        2: begin
            if (pll_saved) begin
                pin_out = shl32(1,ctrapin);
            end
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        3: begin
            pin_out = shl32(1,mux32(pll_saved,ctrapin,ctrbpin));
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        4: begin
            if (testBit64(phase,31)) begin
                pin_out = shl32(1,ctrapin);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        5: begin
            pin_out = shl32(1,mux32(testBit64(phase,31), ctrapin, ctrbpin));
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        6: begin
            if (testBit64(phase,32)) begin
                pin_out = shl32(1,ctrapin);
            end
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        7: begin
            pin_out = shl32(1,mux32(testBit64(phase,32), ctrapin, ctrbpin));
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        8: begin
            if (posedge_clk_cog) begin
                if (delayed0) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        9: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (notB(delayed0)) begin
                pin_out = shl32(1,ctrbpin);
            end
            if (posedge_clk_cog) begin
                if (delayed0) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        10: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                if (andB(delayed0,notB(delayed1))) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        11: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (notB(delayed0)) begin
                pin_out = shl32(1,ctrbpin);
            end
            if (posedge_clk_cog) begin
                if (andB(delayed0,notB(delayed1))) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        12: begin
            if (posedge_clk_cog) begin
                if (notB(delayed0)) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        13: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (notB(delayed0)) begin
                pin_out = shl32(1,ctrbpin);
            end
            if (posedge_clk_cog) begin
                if (notB(delayed0)) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        14: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (posedge_clk_cog) begin
                if (andB(notB(delayed0),delayed1)) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        15: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (notB(delayed0)) begin
                pin_out = or32(pin_out,shl32(1,ctrbpin));
            end
            if (posedge_clk_cog) begin
                if (andB(notB(delayed0),delayed1)) begin
                    phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
                end
                delayed1 = delayed0;
                delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            end
        end
        16: begin
        end
        17: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,andB(notB(delayed1),notB(delayed0)))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        18: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,andB(notB(delayed1),delayed0))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        19: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,notB(delayed1))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        20: begin
            if (andB(posedge_clk_cog,andB(delayed1,notB(delayed0)))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        21: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,notB(delayed0))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        22: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,neqB(delayed1,delayed0))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        23: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,notB(andB(delayed1,delayed0)))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        24: begin
            if (andB(posedge_clk_cog,andB(delayed1,delayed0))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        25: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,eqB(delayed1,delayed0))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        26: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,delayed0)) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        27: begin
            if (posedge_clk_pll) begin
                pllcounter = and64(add64(pllcounter,cast64(frequency)),64'hFFFFFFFFF);
            end
            if (andB(posedge_clk_cog,orB(delayed0,notB(delayed1)))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        28: begin
            if (andB(posedge_clk_cog,delayed1)) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        29: begin
            if (andB(posedge_clk_cog,orB(notB(delayed0),delayed1))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        30: begin
            if (andB(posedge_clk_cog,orB(delayed0,delayed1))) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
        31: begin
            if (posedge_clk_cog) begin
                phase = add64(and64(phase,64'hFFFFFFFF),cast64(frequency));
            end
        end
    endcase
    
    if (geq32(ctrmode,16)) begin
        if (posedge_clk_cog) begin
            delayed0 = eq32(and32(shr32(pin_in,ctrapin),1),1);
            delayed1 = eq32(and32(shr32(pin_in,ctrbpin),1),1);
        end
    end
    
    
    if (posedge_clk_cog) begin
        if (setphs) begin
            phase = cast64(data);
        end
        if (setfrq) begin
            frequency = data;
        end
    end
    
    if (orB(posedge_clk_cog,negedge_ena)) begin
        if (notB(ena)) begin
            //ctrdummy3 = 0;
            //ctrdummy2 = 0;
            //ctrdummy1 = 0;
            ctrmode = 0;
            ctrplldiv = 0;
            ctrbpin = 0;
            ctrapin = 0;
        end else if (setctr) begin
            //ctrdummy3 =     and32(shr32(data,31),   32'h001);
            ctrmode   =     and32(shr32(data,26),   32'h03F);
            ctrplldiv =     and32(shr32(data,23),   32'h007);
            //ctrdummy2 =     and32(shr32(data,14),   32'h1FF);
            ctrbpin   =     and32(shr32(data,9),    32'h01F);
            //ctrdummy1 =     and32(shr32(data,5),    32'h00F);
            ctrapin   =     and32(data,             32'h01F);
        end
    end
    /*@cexp:code-stop*/
    
    
    //Damit die Verifikation richtig ist
    if (orB(posedge_clk_cog,negedge_ena)) begin
        if (notB(ena)) begin
            ctrdummy3 = 0;
            ctrdummy2 = 0;
            ctrdummy1 = 0;
        end else if (setctr) begin
            ctrdummy3 = and32(shr32(data,31),   32'h001);
            ctrdummy2 = and32(shr32(data,14),   32'h1FF);
            ctrdummy1 = and32(shr32(data,5),    32'h00F);
        end
    end
    
    reg_frq_d = frequency;
    reg_phs_d = phase;
    reg_pll_fake_d = pllcounter;
    reg_ctr_d = {ctrdummy3,ctrmode[4:0],ctrplldiv[2:0],ctrdummy2[8:0],ctrbpin[4:0],ctrdummy1[3:0],ctrapin[4:0]};
    reg_dly_d = {delayed1,delayed0};
    
    /*@cexp:always-block-end*/
end


endmodule
