module verifyequal_cog_ctr (
input setfrq,
input posedge_clk_cog,
input [31:0] data,
input [31:0] reg_frq_q,
input [35:0] reg_pll_fake_q,
input posedge_clk_pll,
input [31:0] pin_in,
input ena,
input setphs,
input [31:0] reg_ctr_q,
input negedge_ena,
input [32:0] reg_phs_q,
input clk_pll,
input [1:0] reg_dly_q,
input clk_cog,
input setctr
);
wire [31:0] pin_out_expected;
wire [31:0] pin_out_result;
wire [32:0] reg_phs_d_expected;
wire [63:0] reg_phs_d_result;
wire [35:0] reg_pll_fake_d_expected;
wire [63:0] reg_pll_fake_d_result;
wire [1:0] reg_dly_d_expected;
wire [31:0] reg_dly_d_result;
wire [31:0] reg_ctr_d_expected;
wire [31:0] reg_ctr_d_result;
wire pll_expected;
wire pll_result;
wire [32:0] phs_expected;
wire [63:0] phs_result;
wire [31:0] reg_frq_d_expected;
wire [31:0] reg_frq_d_result;
cog_ctr golden(
    .setfrq (setfrq),
    .posedge_clk_cog (posedge_clk_cog),
    .data (data),
    .pin_out (pin_out_expected),
    .reg_frq_q (reg_frq_q),
    .reg_pll_fake_q (reg_pll_fake_q),
    .posedge_clk_pll (posedge_clk_pll),
    .pin_in (pin_in),
    .ena (ena),
    .setphs (setphs),
    .reg_phs_d (reg_phs_d_expected),
    .reg_ctr_q (reg_ctr_q),
    .reg_pll_fake_d (reg_pll_fake_d_expected),
    .reg_dly_d (reg_dly_d_expected),
    .negedge_ena (negedge_ena),
    .reg_phs_q (reg_phs_q),
    .reg_ctr_d (reg_ctr_d_expected),
    .pll (pll_expected),
    .clk_pll (clk_pll),
    .reg_dly_q (reg_dly_q),
    .clk_cog (clk_cog),
    .phs (phs_expected),
    .setctr (setctr),
    .reg_frq_d (reg_frq_d_expected));
cloned_cog_ctr cloned(
    .setfrq (setfrq),
    .posedge_clk_cog (posedge_clk_cog),
    .data (data),
    .pin_out (pin_out_result),
    .reg_frq_q (reg_frq_q),
    .reg_pll_fake_q (reg_pll_fake_q),
    .posedge_clk_pll (posedge_clk_pll),
    .pin_in (pin_in),
    .ena (ena),
    .setphs (setphs),
    .reg_phs_d (reg_phs_d_result),
    .reg_ctr_q (reg_ctr_q),
    .reg_pll_fake_d (reg_pll_fake_d_result),
    .reg_dly_d (reg_dly_d_result),
    .negedge_ena (negedge_ena),
    .reg_phs_q (reg_phs_q),
    .reg_ctr_d (reg_ctr_d_result),
    .pll (pll_result),
    .clk_pll (clk_pll),
    .reg_dly_q (reg_dly_q),
    .clk_cog (clk_cog),
    .phs (phs_result),
    .setctr (setctr),
    .reg_frq_d (reg_frq_d_result));
assert property(pin_out_result == pin_out_expected);
assert property(reg_phs_d_result[32:0] == reg_phs_d_expected);
assert property(reg_phs_d_result[63:33] == 64'h0);
assert property(reg_pll_fake_d_result[35:0] == reg_pll_fake_d_expected);
assert property(reg_pll_fake_d_result[63:36] == 64'h0);
assert property(reg_dly_d_result[1:0] == reg_dly_d_expected);
assert property(reg_dly_d_result[31:2] == 64'h0);
assert property(reg_ctr_d_result == reg_ctr_d_expected);
assert property(pll_result == pll_expected);
assert property(phs_result[32:0] == phs_expected);
assert property(phs_result[63:33] == 64'h0);
assert property(reg_frq_d_result == reg_frq_d_expected);
endmodule

//-show pin_out_result -show pin_out_expected -show reg_phs_d_result -show reg_phs_d_expected -show reg_pll_fake_d_result -show reg_pll_fake_d_expected -show reg_dly_d_result -show reg_dly_d_expected -show reg_ctr_d_result -show reg_ctr_d_expected -show pll_result -show pll_expected -show phs_result -show phs_expected -show reg_frq_d_result -show reg_frq_d_expected 

module cog_ctr (
input setfrq,
input posedge_clk_cog,
input [31:0] data,
output [31:0] pin_out,
input [31:0] reg_frq_q,
input [35:0] reg_pll_fake_q,
input posedge_clk_pll,
input [31:0] pin_in,
input ena,
input setphs,
output reg [32:0] reg_phs_d,
input [31:0] reg_ctr_q,
output reg [35:0] reg_pll_fake_d,
output reg [1:0] reg_dly_d,
input negedge_ena,
input [32:0] reg_phs_q,
output reg [31:0] reg_ctr_d,
output pll,
input clk_pll,
input [1:0] reg_dly_q,
input clk_cog,
output [32:0] phs,
input setctr,
output reg [31:0] reg_frq_d
);
wire [3:0] pick;
wire outb;
wire outa;
wire trig;
wire [2:0] tba;
wire [2:0] tp[16];
wire [7:0] pll_taps;
assign tp[15] = {(reg_dly_q == 2'b10), (!reg_dly_q[0]), 1'b0};
assign tp[14] = {(reg_dly_q == 2'b10), 1'b0, 1'b0};
assign tp[13] = {(!reg_dly_q[0]), (!reg_dly_q[0]), 1'b0};
assign tp[12] = {(!reg_dly_q[0]), 1'b0, 1'b0};
assign tp[11] = {(reg_dly_q == 2'b1), (!reg_dly_q[0]), 1'b0};
assign tp[10] = {(reg_dly_q == 2'b1), 1'b0, 1'b0};
assign tp[9] = {reg_dly_q[0], (!reg_dly_q[0]), 1'b0};
assign tp[8] = {reg_dly_q[0], 1'b0, 1'b0};
assign tp[7] = {1'b1, (!reg_phs_q[32]), reg_phs_q[32]};
assign tp[6] = {1'b1, 1'b0, reg_phs_q[32]};
assign tp[5] = {1'b1, (!reg_phs_q[31]), reg_phs_q[31]};
assign tp[4] = {1'b1, 1'b0, reg_phs_q[31]};
assign tp[3] = {1'b1, (!pll), pll};
assign tp[2] = {1'b1, 1'b0, pll};
assign tp[1] = {1'b1, 1'b0, 1'b0};
assign tp[0] = {1'b0, 1'b0, 1'b0};
assign pick = reg_ctr_q[29:26];
assign tba = tp[pick];
assign trig = ((reg_ctr_q[30]) ? (pick[reg_dly_q]) : (tba[2]));
assign outb = ((reg_ctr_q[30]) ? (1'b0) : (tba[1]));
assign outa = ((reg_ctr_q[30]) ? (1'b0) : (tba[0]));
assign pin_out = ((outb << reg_ctr_q[13:9]) | (outa << reg_ctr_q[4:0]));
assign pll_taps = reg_pll_fake_q[35:28];
assign pll = pll_taps[(~reg_ctr_q[25:23])];
assign phs = reg_phs_q;
always @(*) begin
    reg_phs_d <= reg_phs_q;
    reg_ctr_d <= reg_ctr_q;
    reg_dly_d <= reg_dly_q;
    reg_frq_d <= reg_frq_q;
    reg_pll_fake_d <= reg_pll_fake_q;
    if ((posedge_clk_cog || negedge_ena)) begin
        if ((!ena)) begin
            reg_ctr_d <= 32'h0;
        end else begin
            if (setctr) begin
                reg_ctr_d <= data;
            end
        end
    end
    if (posedge_clk_cog) begin
        if (setfrq) begin
            reg_frq_d <= data;
        end
    end
    if (posedge_clk_cog) begin
        if ((|reg_ctr_q[30:29])) begin
            reg_dly_d <= {((reg_ctr_q[30]) ? (pin_in[reg_ctr_q[13:9]]) : (reg_dly_q[0])), pin_in[reg_ctr_q[4:0]]};
        end
    end
    if (posedge_clk_cog) begin
        if ((setphs || trig)) begin
            reg_phs_d <= ((setphs) ? ({1'b0, data}) : (({1'b0, reg_phs_q[31:0]} + {1'b0, reg_frq_q})));
        end
    end
    if (posedge_clk_pll) begin
        if (((~&reg_ctr_q[30:28]) && (|reg_ctr_q[27:26]))) begin
            reg_pll_fake_d <= (reg_pll_fake_q + {4'b0, reg_frq_q});
        end
    end
end
endmodule

