module verifyequal_cog_ram (
input [31:0] reg_q_q,
input [8:0] a,
input [16383:0] reg_r_q,
input clk,
input [31:0] d,
input w,
input ena,
input posedge_clk
);
wire [31:0] q_expected;
wire [31:0] q_result;
wire [31:0] reg_q_d_expected;
wire [31:0] reg_q_d_result;
wire [16383:0] reg_r_d_expected;
wire [16383:0] reg_r_d_result;
cog_ram golden(
    .reg_q_q (reg_q_q),
    .a (a),
    .q (q_expected),
    .reg_r_q (reg_r_q),
    .reg_q_d (reg_q_d_expected),
    .clk (clk),
    .reg_r_d (reg_r_d_expected),
    .d (d),
    .w (w),
    .ena (ena),
    .posedge_clk (posedge_clk));
cloned_cog_ram cloned(
    .reg_q_q (reg_q_q),
    .a (a),
    .q (q_result),
    .reg_r_q (reg_r_q),
    .reg_q_d (reg_q_d_result),
    .clk (clk),
    .reg_r_d (reg_r_d_result),
    .d (d),
    .w (w),
    .ena (ena),
    .posedge_clk (posedge_clk));
assert property(q_result == q_expected);
assert property(reg_q_d_result == reg_q_d_expected);
assert property(reg_r_d_result == reg_r_d_expected);
endmodule

//-show q_result -show q_expected -show reg_q_d_result -show reg_q_d_expected -show reg_r_d_result -show reg_r_d_expected 

module cog_ram (
input [31:0] reg_q_q,
input [8:0] a,
output [31:0] q,
input [16383:0] reg_r_q,
output reg [31:0] reg_q_d,
input clk,
output reg [16383:0] reg_r_d,
input [31:0] d,
input w,
input ena,
input posedge_clk
);
assign q = reg_q_q;
always @(*) begin
    reg_r_d <= reg_r_q;
    reg_q_d <= reg_q_q;
    if (posedge_clk) begin
        if ((ena && w)) begin
            reg_r_d[((a * 32'h20) + 32'h0)] <= d[32'h0];
            reg_r_d[((a * 32'h20) + 32'h1)] <= d[32'h1];
            reg_r_d[((a * 32'h20) + 32'h2)] <= d[32'h2];
            reg_r_d[((a * 32'h20) + 32'h3)] <= d[32'h3];
            reg_r_d[((a * 32'h20) + 32'h4)] <= d[32'h4];
            reg_r_d[((a * 32'h20) + 32'h5)] <= d[32'h5];
            reg_r_d[((a * 32'h20) + 32'h6)] <= d[32'h6];
            reg_r_d[((a * 32'h20) + 32'h7)] <= d[32'h7];
            reg_r_d[((a * 32'h20) + 32'h8)] <= d[32'h8];
            reg_r_d[((a * 32'h20) + 32'h9)] <= d[32'h9];
            reg_r_d[((a * 32'h20) + 32'ha)] <= d[32'ha];
            reg_r_d[((a * 32'h20) + 32'hb)] <= d[32'hb];
            reg_r_d[((a * 32'h20) + 32'hc)] <= d[32'hc];
            reg_r_d[((a * 32'h20) + 32'hd)] <= d[32'hd];
            reg_r_d[((a * 32'h20) + 32'he)] <= d[32'he];
            reg_r_d[((a * 32'h20) + 32'hf)] <= d[32'hf];
            reg_r_d[((a * 32'h20) + 32'h10)] <= d[32'h10];
            reg_r_d[((a * 32'h20) + 32'h11)] <= d[32'h11];
            reg_r_d[((a * 32'h20) + 32'h12)] <= d[32'h12];
            reg_r_d[((a * 32'h20) + 32'h13)] <= d[32'h13];
            reg_r_d[((a * 32'h20) + 32'h14)] <= d[32'h14];
            reg_r_d[((a * 32'h20) + 32'h15)] <= d[32'h15];
            reg_r_d[((a * 32'h20) + 32'h16)] <= d[32'h16];
            reg_r_d[((a * 32'h20) + 32'h17)] <= d[32'h17];
            reg_r_d[((a * 32'h20) + 32'h18)] <= d[32'h18];
            reg_r_d[((a * 32'h20) + 32'h19)] <= d[32'h19];
            reg_r_d[((a * 32'h20) + 32'h1a)] <= d[32'h1a];
            reg_r_d[((a * 32'h20) + 32'h1b)] <= d[32'h1b];
            reg_r_d[((a * 32'h20) + 32'h1c)] <= d[32'h1c];
            reg_r_d[((a * 32'h20) + 32'h1d)] <= d[32'h1d];
            reg_r_d[((a * 32'h20) + 32'h1e)] <= d[32'h1e];
            reg_r_d[((a * 32'h20) + 32'h1f)] <= d[32'h1f];
        end
        if (ena) begin
            reg_q_d <= reg_r_q[(a * 32'h20)+31:(a * 32'h20)];
        end
    end
end
endmodule

