// hub

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

/*@cexp:settings:2*/
module cloned_hub
(
/*@cexp:port-ignore*/ input clk_cog,
/*@cexp:port:1*/ input ena_bus,
/*@cexp:port:1*/ input nres,
/*@cexp:port:0*/ input [31:0] bus_sel,
/*@cexp:port:1*/ input bus_r,
/*@cexp:port:1*/ input bus_e,
/*@cexp:port:1*/ input bus_w,
/*@cexp:port:1*/ input [31:0] bus_s,
/*@cexp:port:1*/ input [31:0] bus_a,
/*@cexp:port:1*/ input [31:0] bus_d,
/*@cexp:port:0*/ output reg [31:0] bus_q,
/*@cexp:port:0*/ output reg bus_c,
/*@cexp:port:0*/ output reg [31:0] bus_ack,
/*@cexp:port:0*/ output reg [31:0] cog_ena,
/*@cexp:port:0*/ output reg [31:0] ptr_w,
/*@cexp:port:0*/ output reg [31:0] ptr_d,
/*@cexp:port:1*/ output reg [31:0] cfg,

/*@cexp:port-state*/ input reg_rc_q,
/*@cexp:port-state*/ input reg_ec_q,
/*@cexp:port-state*/ input reg_wc_q,
/*@cexp:port-state*/ input [1:0] reg_sc_q,
/*@cexp:port-state*/ input [15:0] reg_ac_q,
/*@cexp:port-state*/ input [31:0] reg_dc_q,
/*@cexp:port-state*/ input reg_rd_q,
/*@cexp:port-state*/ input reg_ed_q,
/*@cexp:port-state*/ input [1:0] reg_sd_q,
/*@cexp:port-state*/ input [1:0] reg_ad_q,
/*@cexp:port-state*/ input reg_sys_c_q,
/*@cexp:port-state*/ input [2:0] reg_sys_q_q,
/*@cexp:port-state*/ input [7:0] reg_cog_e_q,
/*@cexp:port-state*/ input [7:0] reg_lock_e_q,
/*@cexp:port-state*/ input [7:0] reg_lock_state_q,
/*@cexp:port-state*/ input [7:0] reg_cfg_q,
/*@cexp:port-state*/ input [7:0] reg_cog_ena_q,
/*@cexp:port-state*/ input [31:0] reg_bus_q_q,
/*@cexp:port-state*/ output reg reg_rd_d,
/*@cexp:port-state*/ output reg reg_ed_d,
/*@cexp:port-state*/ output reg [31:0] reg_sd_d,
/*@cexp:port-state*/ output reg [31:0] reg_ad_d,
/*@cexp:port-state*/ output reg reg_rc_d,
/*@cexp:port-state*/ output reg reg_ec_d,
/*@cexp:port-state*/ output reg reg_wc_d,
/*@cexp:port-state*/ output reg [31:0] reg_sc_d,
/*@cexp:port-state*/ output reg [31:0] reg_ac_d,
/*@cexp:port-state*/ output reg [31:0] reg_dc_d,
/*@cexp:port-state*/ output reg reg_sys_c_d,
/*@cexp:port-state*/ output reg [31:0] reg_sys_q_d,
/*@cexp:port-state*/ output reg [31:0] reg_cog_e_d,
/*@cexp:port-state*/ output reg [31:0] reg_lock_e_d,
/*@cexp:port-state*/ output reg [31:0] reg_lock_state_d,
/*@cexp:port-state*/ output reg [31:0] reg_cfg_d,
/*@cexp:port-state*/ output reg [31:0] reg_cog_ena_d,
/*@cexp:port-state*/ output reg [31:0] reg_bus_q_d,

//Untermodul hub_mem
/*@cexp:submodule:hub_mem:hub_mem*/
/*@cexp:port-submodule-dummy*/ output reg submod_hub_mem_clk_cog_in,
/*@cexp:port-submodule:1:hub_mem:0*/ output reg submod_hub_mem_ena_bus_in,
/*@cexp:port-submodule:1:hub_mem:0*/ output reg submod_hub_mem_w_in,
/*@cexp:port-submodule:1:hub_mem:0*/ output reg [31:0] submod_hub_mem_wb_in,
/*@cexp:port-submodule:1:hub_mem:0*/ output reg [31:0] submod_hub_mem_a_in,
/*@cexp:port-submodule:1:hub_mem:0*/ output reg [31:0] submod_hub_mem_d_in,
/*@cexp:port-submodule:1:hub_mem:0*/ input [31:0] submod_hub_mem_q_out,


//Taktflanken Signale
/*@cexp:port-clock:1*/ input posedge_clk_cog,
/*@cexp:port-clock:1*/ input negedge_nres
);


//Interne Register
/*@cexp:reg-state:1:1*/ reg [31:0] sd;
/*@cexp:reg-state:1:1*/ reg [31:0] ad;
/*@cexp:reg-state:0,1:1*/ reg [31:0] sc;
/*@cexp:reg-state:0,1:1*/ reg [31:0] ac;
/*@cexp:reg-state:0,1:1*/ reg [31:0] dc;
/*@cexp:reg-state:1:1*/ reg [31:0] sys_q;
/*@cexp:reg-state:0,1:1*/ reg [31:0] cog_e;
/*@cexp:reg-state:0,1:1*/ reg [31:0] lock_e;
/*@cexp:reg-state:1:1*/ reg [31:0] lock_state;
/*@cexp:reg-state:1:1*/ reg [31:0] cfg_reg;
/*@cexp:reg-state:0,1:1*/ reg [31:0] cog_ena_reg;
/*@cexp:reg-state:0,1:1*/ reg [31:0] bus_q_reg;
/*@cexp:reg-state:0,1:0*/ reg [31:0] bus_sel_saved;
/*@cexp:reg-state:0,1:0*/ reg [31:0] enc;
/*@cexp:reg-state:0,1:0*/ reg [31:0] num;
/*@cexp:reg-state:0,1:0*/ reg [31:0] num_dcd;
/*@cexp:reg-state:0,1:0*/ reg [31:0] cog_start;
/*@cexp:reg-state:1:1*/ reg rd;
/*@cexp:reg-state:0,1:1*/ reg ed;
/*@cexp:reg-state:1:1*/ reg rc;
/*@cexp:reg-state:0,1:1*/ reg ec;
/*@cexp:reg-state:1:1*/ reg wc;
/*@cexp:reg-state:0,1:1*/ reg sys_c;
/*@cexp:reg-state:0,1:0*/ reg sys;


//Lokale Variablen
/*@cexp:reg-local:1*/ reg [31:0] ramq;
/*@cexp:reg-local:1*/ reg [31:0] ac0_num_dcd;
/*@cexp:reg-local:1*/ reg [31:0] hb_din;
/*@cexp:reg-local:1*/ reg [31:0] hb_wb;


function [31:0] decrypt;
    input [31:0] indata;
    begin
        // unscramble rom data if cog loading
        decrypt = 
or32(or32(or32(
or32(or32(shl32(and32(shr32(indata,03),1),31), shl32(and32(shr32(indata,07),1),30)), or32(shl32(and32(shr32(indata,21),1),29), shl32(and32(shr32(indata,12),1),28))),
or32(or32(shl32(and32(shr32(indata,06),1),27), shl32(and32(shr32(indata,19),1),26)), or32(shl32(and32(shr32(indata,04),1),25), shl32(and32(shr32(indata,17),1),24)))),

or32(
or32(or32(shl32(and32(shr32(indata,20),1),23), shl32(and32(shr32(indata,15),1),22)), or32(shl32(and32(shr32(indata,08),1),21), shl32(and32(shr32(indata,11),1),20))),
or32(or32(shl32(and32(shr32(indata,00),1),19), shl32(and32(shr32(indata,14),1),18)), or32(shl32(and32(shr32(indata,30),1),17), shl32(and32(shr32(indata,01),1),16))))),

or32(or32(
or32(or32(shl32(and32(shr32(indata,23),1),15), shl32(and32(shr32(indata,31),1),14)), or32(shl32(and32(shr32(indata,16),1),13), shl32(and32(shr32(indata,05),1),12))),
or32(or32(shl32(and32(shr32(indata,09),1),11), shl32(and32(shr32(indata,18),1),10)), or32(shl32(and32(shr32(indata,25),1), 9), shl32(and32(shr32(indata,02),1), 8)))),

or32(
or32(or32(shl32(and32(shr32(indata,28),1), 7), shl32(and32(shr32(indata,22),1), 6)), or32(shl32(and32(shr32(indata,13),1), 5), shl32(and32(shr32(indata,27),1), 4))),
or32(or32(shl32(and32(shr32(indata,29),1), 3), shl32(and32(shr32(indata,24),1), 2)), or32(shl32(and32(shr32(indata,26),1), 1), shl32(and32(shr32(indata,10),1), 0))))));
    end
endfunction

always @(*) begin
    /*@cexp:dummyinit-start*/
    rd = 0;
    ramq = 0;
    ed = 0;
    cfg = 0;
    cog_ena = 0;
    bus_q = 0;
    bus_ack = 0;
    sd = 0;
    ad = 0;
    rc = 0;
    ec = 0;
    wc = 0;
    sc = 0;
    ac = 0;
    dc = 0;
    sys_c = 0;
    sys_q = 0;
    cog_e = 0;
    lock_e = 0;
    lock_state = 0;
    cfg_reg = 0;
    cog_ena_reg = 0;
    bus_q_reg = 0;
    bus_c = 0;
    ptr_w = 0;
    ptr_d = 0;
    enc = 0;
    ac0_num_dcd = 0;

    submod_hub_mem_clk_cog_in = 0;
    submod_hub_mem_ena_bus_in = 0;
    submod_hub_mem_w_in = 0;
    submod_hub_mem_wb_in = 0;
    submod_hub_mem_a_in = 0;
    submod_hub_mem_d_in = 0;
    sys = 0;
    num = 0;
    num_dcd = 0;
    cog_start = 0;
    
    reg_rd_d = 0;
    reg_ed_d = 0;
    reg_sd_d = 0;
    reg_ad_d = 0;
    reg_rc_d = 0;
    reg_ec_d = 0;
    reg_wc_d = 0;
    reg_sc_d = 0;
    reg_ac_d = 0;
    reg_dc_d = 0;
    reg_sys_c_d = 0;
    reg_sys_q_d = 0;
    reg_cog_e_d = 0;
    reg_lock_e_d = 0;
    reg_lock_state_d = 0;
    reg_cfg_d = 0;
    reg_cog_ena_d = 0;
    reg_bus_q_d = 0;
    bus_sel_saved = 0;
    hb_din=0;
    hb_wb=0;
    /*@cexp:dummyinit-stop*/

    rd = reg_rd_q;
    ed = reg_ed_q;
    sd = reg_sd_q;
    ad = reg_ad_q;
    rc = reg_rc_q;
    ec = reg_ec_q;
    wc = reg_wc_q;
    sc = reg_sc_q;
    ac = reg_ac_q;
    dc = reg_dc_q;
    sys_c = reg_sys_c_q;
    sys_q = reg_sys_q_q;
    cog_e = reg_cog_e_q;
    lock_e = reg_lock_e_q;
    lock_state = reg_lock_state_q;
    cfg_reg = reg_cfg_q;
    cog_ena_reg = reg_cog_ena_q;
    bus_q_reg = reg_bus_q_q;

    
    submod_hub_mem_clk_cog_in = clk_cog; //TODO wie mit clock verfahren?
    
    //Hier geht es los
    /*@cexp:code-start:*/
    bus_q = bus_q_reg;
    bus_c = sys_c;
    ptr_d = shr32(dc,4);
    cog_ena = cog_ena_reg;
    bus_ack = mux32(ed,or32(shl32(and32(bus_sel,3),6),shr32(bus_sel,2)),0);
    bus_sel_saved = bus_sel;
    
    enc = mux32(testBit32(ac,2),lock_e,cog_e);
    num = mux32(orB(andB(eq32(and32(ac,7),3'b010),testBit32(dc,3)),eq32(and32(ac,7),3'b100)),
            mux32(eq32(and32(enc,32'h7F),32'h7F),3'b111, // x1111111 -> 111
            mux32(eq32(and32(enc,32'h7F),32'h3F),3'b110, // x0111111 -> 110
            mux32(eq32(and32(enc,32'h3F),32'h1F),3'b101, // xx011111 -> 101
            mux32(eq32(and32(enc,32'h1F),32'h0F),3'b100, // xxx01111 -> 100
            mux32(eq32(and32(enc,32'h0F),32'h07),3'b011, // xxxx0111 -> 011
            mux32(eq32(and32(enc,32'h07),32'h03),3'b010, // xxxxx011 -> 010
            mux32(eq32(and32(enc,32'h03),32'h01),3'b001, // xxxxxx01 -> 001
            3'b000))))))), // xxxxxxx0 -> 000
                        and32(dc,7));
    num_dcd = shl32(1'b1,num);
    sys	= andB(ec,eq32(sc,3));
    cog_start = and32(mux32(andB(andB(sys,eq32(and32(ac,7),3'b010)),notB(andB(testBit32(dc,3),eq32(enc,32'hFF)))),32'hFF,0),num_dcd);
    ptr_w = cog_start;
    /*@cexp:code-stop*/
    /*@cexp:code-start:clocks_hub*/
    cfg = cfg_reg;
    
    
    
    hb_wb = 4'b1111;
    hb_din = dc;
    if (eq32(and32(sc,3),1)) begin // wrword
        hb_din = and32(dc,32'hFFFF);
        hb_din = or32(hb_din,shl32(hb_din,16));
        hb_wb = mux32(testBit32(ac,1),4'b1100,4'b0011);
    end else if (eq32(and32(sc,3),0)) begin // wrbyte
        hb_din = and32(dc,32'hFF);
        hb_din = or32(hb_din,shl32(hb_din,8));
        hb_din = or32(hb_din,shl32(hb_din,16));
        hb_wb = shl32(4'b0001,and32(ac,3));
    end
    
    /*@cexp:declare-module-input:hub_mem:hub_mem:0*/
    submod_hub_mem_ena_bus_in = ena_bus;
    submod_hub_mem_w_in = andB(ec,andB(wc,neq32(sc,3)));
    submod_hub_mem_a_in = shr32(ac,2);
    submod_hub_mem_wb_in = hb_wb;
    submod_hub_mem_d_in = hb_din;
    /*@cexp:call-module:hub_mem:hub_mem:0:clockfwd*/
    
    
    if (negedge_nres) begin
        ec = 1'b0;
        ed = 1'b0;
        cfg_reg = 8'b0;
        cog_e = 8'b00000001;
        lock_e = 8'b0;
        cog_ena_reg = 8'b0;
    end
    
    
    if (posedge_clk_cog) begin
        ac0_num_dcd = and32(mux32(testBit32(ac,0),0,32'hFF),num_dcd);
        if (notB(nres)) begin
            ec = 1'b0;
            ed = 1'b0;
            cfg_reg = 8'b0;
            cog_e = 8'b00000001;
            lock_e = 8'b0;
            cog_ena_reg = 8'b0;
        end else if (ena_bus) begin
            ed = ec;
            ec = bus_e;
            cog_ena_reg = and32(cog_e,invert32(cog_start));
            if (sys) begin
                if (eq32(and32(ac,7),0)) begin
                    cfg_reg = and32(dc,32'hFF);
                end
                if (eq32(and32(ac,6),2)) begin
                    cog_e = or32(and32(cog_e,invert32(num_dcd)),ac0_num_dcd);
                end
                if (eq32(and32(ac,6),4)) begin
                    lock_e = or32(and32(lock_e,invert32(num_dcd)),ac0_num_dcd);
                end
            end
        end
        
        ramq = mux32(notB(rd),submod_hub_mem_q_out,decrypt(submod_hub_mem_q_out));
        
        case(and32(sd,3))
            32'h0: begin
                bus_q_reg = and32(shr32(ramq,shl32(and32(ad,3),3)),32'h000000FF); // rdbyte
            end
            1: begin
                bus_q_reg = and32(shr32(ramq,shl32(and32(ad,2),3)),32'h0000FFFF); // rdword
            end
            2: begin
                bus_q_reg = ramq; // rdlong
            end
            3: begin
                bus_q_reg = sys_q; // cogid/coginit/locknew
            end
        endcase
        
    
    
        if (ena_bus) begin
            rd = andB(notB(rc),testBit32(ac,15));
            rc = bus_r;
            wc = bus_w;
            sd = sc;
            sc = bus_s;
            ad = and32(ac,3);
            if (sys) begin
                
                if (eq32(and32(ac,6),6)) begin
                    sys_q = num;                                                              // others
                    sys_c = testBit32(shr32(lock_state,and32(dc,7)),0); // lockset/lockclr
                    lock_state = or32(and32(lock_state,invert32(num_dcd)),ac0_num_dcd);
                end else begin
                    if (eq32(and32(ac,7),1)) begin
                        sys_q = 0;
                        if (neq32(and32(bus_sel_saved,32'hE1),0)) begin
                            sys_q = 4;
                        end
                        if (neq32(and32(bus_sel_saved,32'h99),0)) begin
                            sys_q = or32(sys_q,2);
                        end
                        if (neq32(and32(bus_sel_saved,32'h55),0)) begin
                            sys_q = or32(sys_q,1);
                        end
                    end else begin
                        sys_q = num;
                    end
                    sys_c = eq32(enc,32'hFF);  // others
                end
            end
            dc = bus_d;
            ac = bus_a;
        end
    end
    
    /*@cexp:code-stop*/
    
    //Register Update
    reg_rd_d = rd;
    reg_ed_d = ed;
    reg_sd_d = sd;
    reg_ad_d = ad;
    reg_rc_d = rc;
    reg_ec_d = ec;
    reg_wc_d = wc;
    reg_sc_d = sc;
    reg_ac_d = ac;
    reg_dc_d = dc;
    reg_sys_c_d = sys_c;
    reg_sys_q_d = sys_q;
    reg_cog_e_d = cog_e;
    reg_lock_e_d = lock_e;
    reg_lock_state_d = lock_state;
    reg_cfg_d = cfg_reg;
    reg_cog_ena_d = cog_ena_reg;
    reg_bus_q_d = bus_q_reg;
    /*@cexp:always-block-end*/
end


// connect hub memory to signals from cog[n-1]


// sys common logic
//
//	ac in				dc in				num				sys_q			sys_c
//	-----------------------------------------------------------------------------------------
//	000		CLKSET		config(8)			-				-				-
//	001		COGID		-					-				[n-1]			-
//	010		COGINIT		ptr(28),newx,id(3)	id(3)/newx		id(3)/newx		all
//	011		COGSTOP		-,id(3)				id(3)			id(3)			-
//	100		LOCKNEW		-					newx			newx			all
//	101		LOCKRET		-,id(3)				id(3)			id(3)			-
//	110		LOCKSET		-,id(3)				id(3)			id(3)			lock_state[id(3)]
//	111		LOCKCLR		-,id(3)				id(3)			id(3)			lock_state[id(3)]

endmodule
