// hub_mem

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

module				cloned_hub_mem
(
input				clk_cog,
input				ena_bus,

input				w,
input		 [3:0]	wb,
input		[13:0]	a,
input		[31:0]	d,

output		[31:0]	q
);


// 8192 x 32 ram with byte-write enables ($0000..$7FFF)
reg [31:0] reg_q;


assign q = reg_q;

always @(posedge clk_cog)
begin
    if (ena_bus) begin
        const quint32 amasked <= and32(a,32'h3FFF);
        
        if (andB(w,notB(testBit32(amasked,13)))) begin
            quint32 newMask <= or32(
                        or32(
                            mux32(testBit32(wb,0),32'hFF,0),
                            mux32(testBit32(wb,1),32'hFF00,0)
                        ),
                        or32(
                            mux32(testBit32(wb,2),32'hFF0000,0),
                            mux32(testBit32(wb,3),32'hFF000000,0)
                            )
                        );
            memory[amasked] <= or32(and32(d,newMask),and32(memory[amasked],invert32(newMask)));
        end
        reg_q <= memory[amasked];
    end
end

endmodule
