#include "abstractpropeller.h"

QString CogStatus::stateName() const {
    QString res;
    switch(state) {
        case StateNone: res = "0/NON"; break;
        case StateRdS: res = "1/RdS"; break;
        case StateRdD: res = "2/RdD"; break;
        case StateRdI: res = "3/RdI"; break;
        case StateWrD: res = "4/WrD"; break;
        case StateWai: res = "5/Wai"; break;
        case StateIllg: res = "?/ILL"; break;
    }
    return res;
}

bool CogCounterStatus::operator==(const CogCounterStatus& other) const {
    return phase == other.phase && pllcounter == other.pllcounter && frequency == other.frequency
            && ctrmode == other.ctrmode && ctrplldiv == other.ctrplldiv && ctrapin == other.ctrapin
            && ctrbpin == other.ctrbpin && delayed0 == other.delayed0 && delayed1 == other.delayed1;
}

bool CogCounterStatus::operator!=(const CogCounterStatus& other) const {
    return !operator==(other);
}

QString CogCounterStatus::toString() const {
    return QString("phase %1, pllcounter %2, frequency %3, ctrmode %4, ctrplldiv %5, ctrapin %6, ctrbpin %7, delayed0 %8, delayed1 %9")
    .arg(phase)
    .arg(pllcounter)
    .arg(frequency)
    .arg(ctrmode)
    .arg(ctrplldiv)
    .arg(ctrapin)
    .arg(ctrbpin)
    .arg(delayed0)
    .arg(delayed1);
}

AbstractPropeller::AbstractPropeller(QString deviceId, QString deviceName):
    HWTraceable (deviceId, deviceName),
    m_clockTrace(nullptr)
{

}

TraceInfo AbstractPropeller::getDeviceTraceInfo() const {
    return HWTraceable::getDeviceTraceInfo().addMethod("ClockCycle");
}

void AbstractPropeller::setDebugTraceFunction(QString method, HWTraceMethod func) {
    if (method == "ClockCycle")
        m_clockTrace = func;
}

void AbstractPropeller::clockCycle() {
    if (m_clockTrace)
        m_clockTrace("Cycle",0,0);
}
