#ifndef ABSTRACTPROPELLER_H
#define ABSTRACTPROPELLER_H

#include "hwtrace.h"

struct CogStatus {
    enum State {StateNone=0,StateRdS=1,StateRdD=2,StateRdI=3,StateWrD=4,StateWai=5,StateIllg=-1};
    CogStatus():programCounter(0),state(StateIllg),enabled(false),running(false),carryFlag(false),zeroFlag(false) {}
    QString stateName() const;


    int programCounter;
    State state;
    bool enabled;
    bool running;
    bool carryFlag;
    bool zeroFlag;
};

struct CogCounterStatus {
    CogCounterStatus():phase(0),pllcounter(0),frequency(0),ctrmode(0),ctrplldiv(0),ctrapin(0),ctrbpin(0),delayed0(false),delayed1(false) {}
    bool operator==(const CogCounterStatus& other) const;
    bool operator!=(const CogCounterStatus& other) const;
    QString toString() const;
    quint64 phase;
    quint64 pllcounter;
    quint32 frequency;
    quint32 ctrmode;
    quint32 ctrplldiv;
    quint32 ctrapin;
    quint32 ctrbpin;
    bool delayed0;
    bool delayed1;
};

class AbstractPropeller : public HWTraceable
{
public:
    AbstractPropeller(QString deviceId, QString deviceName);
    virtual void reset()=0;
    virtual void clockCycle()=0;

    virtual void setPinIn(quint32 pins)=0;
    virtual quint32 pinOut() const=0;
    virtual quint32 pinDir() const=0;

    virtual CogStatus cogStatus(int cogNo) const=0;
    virtual CogCounterStatus cogCounterStatus(int cogNo, int counterNo)=0;
    virtual int cogMemory(int cogId, int address) const=0;
    virtual int hubMemory(int address) const=0;
    virtual quint32 globalCounter() const=0;

    virtual TraceInfo getDeviceTraceInfo() const;
    virtual void setDebugTraceFunction(QString method, HWTraceMethod func);

    virtual QList<QPair<QString,QString> > listInnerCogVariables(int cogNo) const=0;
    virtual QList<QPair<QString,QString> > listInnerHubVariables() const=0;

    virtual QString name() const=0;
protected:
    HWTraceMethod m_clockTrace;
};

#endif // ABSTRACTPROPELLER_H
