#include "cogramannotation.h"
#include "filetools.h"
#include <QRegExp>
#include <QtDebug>

CogRamAnnotation::CogRamAnnotation():
    annotations(512)
{

}

void CogRamAnnotation::clear() {
    for (int i=0; i<annotations.size(); ++i)
        annotations[i] = Annotation();
    name = "";
}

void CogRamAnnotation::loadFile(QString fileName) {
    clear();
    name = "unnamed";
    QRegExp locationRe("^([a-z]+):([0-9]+):(.*)$");
    foreach (auto line, FileTools::readTextFile(fileName).split("\n")) {
        line = line.trimmed();
        if (!locationRe.exactMatch(line)) {
            if (line.startsWith("name:"))
                name = line.mid(5);
            continue;
        }
        int idx = locationRe.cap(2).toInt();
        if (idx<0 || idx>=annotations.size())
            continue;
        auto& ann = annotations[idx];
        const QString type = locationRe.cap(1);
        const QString value = locationRe.cap(3);
        if (type == "label")
            ann.label = value;
        else if (type == "comment")
            ann.comment = value;
        else if (type == "extracomment")
            ann.prependComments << value;
        else if (type == "noinstruction")
            ann.noInstruction = true;
        else if (type == "srcnumber")
            ann.forceSrcNumber = true;
        else if (type == "stepover")
            ann.stepOver = true;
    }
}

void CogRamAnnotation::saveFile(QString fileName) {
    QString str = "name:"+name+"\n";
    for (int i=0; i<annotations.size(); ++i) {
        const auto& an(annotations[i]);
        if (!an.label.isEmpty())
            str += QString("label:%1:%2\n").arg(i).arg(an.label);
        if (!an.comment.isEmpty())
            str += QString("comment:%1:%2\n").arg(i).arg(an.comment);
        foreach(QString extraComment, an.prependComments)
            str += QString("extracomment:%1:%2\n").arg(i).arg(extraComment);
        if (an.noInstruction)
            str += QString("noinstruction:%1:\n").arg(i);
        if (an.forceSrcNumber)
            str += QString("srcnumber:%1:\n").arg(i);
        if (an.stepOver)
            str += QString("stepover:%1:\n").arg(i);
    }
    FileTools::saveTextFile(fileName, str);
}
