#include "hubviewer.h"
#include "ui_hubviewer.h"
#include "cogviewer.h"

HubViewer::HubViewer(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::HubViewer),
    m_propeller(nullptr)
{
    ui->setupUi(this);
    for (int i=0; i<MemSize; ++i)
        m_lastMemory[i]=0;
}

HubViewer::~HubViewer() {
    delete ui;
}

void HubViewer::setPropeller(AbstractPropeller *propeller) {
    m_propeller = propeller;
    for (int i=0; i<MemSize; ++i)
        m_lastMemory[i]=0;
    updateState();
}

void HubViewer::updateState() {
    if (!m_propeller) {
        ui->lstMemory->clear();
        ui->lstInnerVars->clear();
        return;
    }
    const int LongsPerLine=4;
    for (int i=0,itemIdx=0; i<MemSize; i+=LongsPerLine,itemIdx++) {
        bool modified = false;
        if (itemIdx>=ui->lstMemory->count()) {
            ui->lstMemory->addItem("");
            ui->lstMemory->item(itemIdx)->setTextColor(QColor(255,255,255));
            ui->lstMemory->item(itemIdx)->setBackgroundColor(QColor(64,64,64));
            modified = true;
        }
        for (int j=0; j<LongsPerLine; ++j)
            if (m_lastMemory[i+j] != m_propeller->hubMemory(i+j))
                modified = true;
        if (!modified)
            continue;
        QString line = "0x"+QString::number(i,16).rightJustified(8,'0')+": ";
        for (int j=0; j<LongsPerLine; ++j)
            line += " "+QString::number(quint32(m_propeller->hubMemory(i+j)),16).rightJustified(8,'0');
        ui->lstMemory->item(itemIdx)->setText(line);
    }
    CogViewer::displayInnerVars(m_propeller->listInnerHubVariables(), ui->lstInnerVars);
}
