#ifndef HWTRACE_H
#define HWTRACE_H

#include <QList>
#include <QString>
#include <QStringList>
#include <QDataStream>
#include <functional>

struct TraceInfo {
    TraceInfo() {}
    TraceInfo addMethod(QString method) const {
        TraceInfo ti(*this);
        ti.traceMethods<<method;
        return ti;
    }
    QString deviceId;
    QString deviceName;
    QStringList traceMethods;

};

typedef std::function<void(const char* text, int data1, int data2)> HWTraceMethod;

struct HWSerializeException {
    HWSerializeException(QString message);
    QString message;
};

class HWTraceable : public QObject
{
    Q_OBJECT
public:
    HWTraceable(QString deviceId, QString deviceName);
    virtual ~HWTraceable();
    virtual TraceInfo getDeviceTraceInfo() const; //nur für das aktuelle Device
    virtual void setDebugTraceFunction(QString method, HWTraceMethod func);
    QString deviceId() const {return m_deviceId; }
    virtual void serialize(QDataStream& stream) const=0;
    virtual void deserialize(QDataStream& stream)=0;
private:
    enum {SerialMagic=0x7BF20533};
    QString m_deviceId;
    QString m_deviceName;
};

#define TRACE(method,text,data1,data2) if (method) method(((text)),((data1)),((data2)))

#endif // HWTRACE_H
