#ifndef I2CDEVICE_H
#define I2CDEVICE_H

#include "hwtrace.h"

class I2CDevice {
public:
    virtual ~I2CDevice();
    virtual bool start(int address)=0; //gibt true zurück, wenn die adresse akzeptiert wird
    virtual bool writeToDevice(int data)=0; //gibt true zurück, wenn ack akzeptiert wird
    virtual int readFromDevice(bool lastByteAck)=0; //lastByteAck gibt den gelesenen Ack Status an (true beim 1. Byte); return gibt das nächste byte zurück
    virtual void stop()=0;
};

class I2CEndpoint : public HWTraceable {
public:
    I2CEndpoint(QString deviceId, QString deviceName, int sclPin, int sdaPin);
    quint32 clockCycle(quint32 pulledUpPins);
    void addDevice(I2CDevice *device);

    //von HWTraceable
    virtual TraceInfo getDeviceTraceInfo() const;
    virtual void setDebugTraceFunction(QString method, HWTraceMethod func);
    virtual void serialize(QDataStream& stream) const;
    virtual void deserialize(QDataStream& stream);
private:
    enum I2CState {Idle, Started, ReadAddress, WriteToDevice, StartReadFromDevice, ReadFromDevice, Cancelled };

    void stop();

    QList<I2CDevice*> m_devices;
    HWTraceMethod m_trace;
    const quint32 m_sclMask;
    const quint32 m_sdaMask;
    const quint32 m_nonSdaMask;
    int m_activeDeviceIndex;
    int m_bitCounter;
    int m_readDataShiftRegister;
    int m_writeDataShiftRegister;
    I2CState m_state;
    bool m_lastScl;
    bool m_lastSdaIn;
    bool m_sdaOut;
};

#endif // I2CDEVICE_H
