#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "simulationmaster.h"
#include <functional>
#include "filetools.h"
#include "propellerwidget.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();
private slots:
    void updateViews();
    void on_actionSingleClockCycle_triggered();
    void on_actionNextAsmInstruction_triggered();
    void on_actionRunUntilBreakpoint_triggered();
    void on_actionLoad_Snapshot_triggered();
    void on_actionSave_Snapshot_triggered();
    void on_spbCogNo_valueChanged(int);
    void on_actionBreak_triggered();
    void on_actionRun_until_global_counter_triggered();
private:
    void loadSnapshot(QString file);
    void setRunning(bool running);
    void setupDebugTracer(std::function<HWTraceMethod(QString deviceId, QString methodName)> callback);
    void triggerBreakpoint();
    void runUntilBreakpoint();
    HWTraceMethod generateDebugMethod(QString deviceId);
    Ui::MainWindow *ui;
    SimulationMaster* m_sim;
    SimulationMaster* m_simGolden;
    PropellerWidget *m_simCompareWidget;
    bool m_breakpointReached;
};

#endif // MAINWINDOW_H
