#include "pdisasm.h"

QStringList PDisAsm::s_mnemonics = QStringList()
          <<"W/R BYTE"<<"W/R WORD"<<"W/R LONG"<<"SWYSOP"<<"<MUL>"<<"<MULS>"<<"<ENC>"<<"<ONES>"
         <<"ROR"<<"ROL"<<"SHR"<<"SHL"<<"RCR"<<"RCL"<<"SAR"<<"REV"
         <<"MINS"<<"MAXS"<<"MIN"<<"MAX"<<"MOVS"<<"MOVD"<<"MOVI"<<"JMPRET"
         <<"AND"<<"ANDN"<<"OR"<<"XOR"<<"MUXC"<<"MUXNC"<<"MUXZ"<<"MUXNZ"
         <<"ADD"<<"SUB"<<"ADDABS"<<"SUBABS"<<"SUMC"<<"SUMNC"<<"SUMZ"<<"SUMNZ"
         <<"MOV"<<"NEG"<<"ABS"<<"ABSNEG"<<"NEGC"<<"NEGNC"<<"NEGZ"<<"NEGNZ"
         <<"CMPS"<<"CMPSX"<<"ADDX"<<"SUBX"<<"ADDS"<<"SUBS"<<"ADDSX"<<"SUBSX"
         <<"CMPSUB"<<"DJNZ"<<"TJNZ"<<"TJZ"<<"WAITPEQ"<<"WAITPNE"<<"WAITCNT"<<"WAITVID"
        ;

QStringList PDisAsm::s_conditions = QStringList()
        <<"never"
        <<"if_nc_and_nz"
        <<"if_nc_and_z"
        <<"if_nc"
        <<"if_c_and_nz"
        <<"if_nz"
        <<"if_c_ne_z"
        <<"if_nc_or_nz"
        <<"if_c_and_z"
        <<"if_c_eq_z"
        <<"if_z"
        <<"if_nc_or_z"
        <<"if_c"
        <<"if_c_or_nz"
        <<"if_c_or_z"
        <<"";


QString PDisAsm::disassemble(const CogRamAnnotation &annotation, int pos, bool isAtPc, int fullInstruction) {
    if (annotation.annotations[pos].noInstruction && !isAtPc)
        return "";
    const int src = fullInstruction & 0x1FF;
    const int dst = (fullInstruction>>9) & 0x1FF;
    const int cond = (fullInstruction>>18) & 0xF;
    const bool immediate = (fullInstruction & (1<<22)) != 0;
    const bool wr = (fullInstruction & (1<<23)) != 0;
    const bool wc = (fullInstruction & (1<<24)) != 0;
    const bool wz = (fullInstruction & (1<<25)) != 0;
    const int opcode = (fullInstruction>>26)&0x3F;
    QString result = s_conditions[cond].leftJustified(13);

    QString mainOp = s_mnemonics[opcode]+" ";
    if (annotation.annotations[dst].label.isEmpty())
        mainOp += "$"+QString::number(dst,16);
    else
        mainOp += annotation.annotations[dst].label;
    mainOp += ", ";
    if (immediate)
        mainOp += "#";

    if (annotation.annotations[pos].forceSrcNumber || annotation.annotations[src].label.isEmpty())
        mainOp += "$"+QString::number(src,16);
    else
        mainOp += annotation.annotations[src].label;

    result += mainOp.leftJustified(25);

    if (wc)
        result += "WC ";
    if (wz)
        result += "WZ ";
    if (!wr)
        result += "NR ";

    return result;
}
