%macro addJumpTableEntry 2
        ;Parameter: Label des Tabellenanfangs, Ziel Label
        dd %2-%1
%endmacro

;Der nun folgende Code wird nicht ausgeführt, sondern bis zum 1. Parameter an
;DST64 kopiert
;Danach wird zum 1. Parameter gesprungen
%macro copyFollowingCodeUntil 1
    lea SRC64,[rel %%startOfCode]
    mov CNT32,%1-%%startOfCode
    rep movsb
    jmp %1
    %%startOfCode:
%endmacro

;Der nun folgende Code wird nicht ausgeführt, sondern bis zum 1. Parameter an
;DST64 kopiert
;Danach wird zum Aufrufer zurückgekehrt
%macro copyFollowingCodeAndRet 1
    lea SRC64,[rel %%startOfCode]
    mov CNT32,%1-%%startOfCode
    rep movsb
    ret
    %%startOfCode:
%endmacro


;Erzeugt im dynamischen Code eine and A32,# Instruktion, wobei #
;hier im A32 register übergeben wird
%macro generateAnd32AInstruction 0
    ;Codiert: 25 ## ## ## ##        and eax,0x#
    mov [rdi],byte 0x25
    mov [rdi+1],dword A32
    add rdi,5
%endmacro

;Erzeugt im dynamischen Code eine shl A32,# Instruktion, wobei #
;hier im B32 register übergeben wird
;das A32 register wird zerstört
%macro generateShl32AInstruction 0
    ;Codiert C1 E0 0C            shl eax,byte 0xc
    mov [rdi],word 0xE0C1
    mov [rdi+2],byte A8
    add rdi,3
%endmacro

;Erzeugt im dynamischen Code eine shr A32,# Instruktion, wobei #
;hier im B32 register übergeben wird
;das A32 register wird zerstört
%macro generateShr32AInstruction 0
    ;Codiert C1 E8 #            shr eax,byte #
    mov [rdi],word 0xE8C1
    mov [rdi+2],byte A8
    add rdi,3
%endmacro

;Parameter 1 ist die Tabelle
;in A64 ist die Eintragsnummer der folgenden Jump Table angegeben
;B64 wird zerstört
%macro executeJumpTable 1
    lea B64,[rel %1]
    mov A32,dword [B64+A64*4]
    add A64,B64
    jmp A64
%endmacro

funCopyFromTemplateToScratchpad:
        ;Kopiert das ausgewählte Template von der relativen Quelle an das Ziel
        ;Das Template kann dabei in verschiedenen, durchnummerierten Versionen vorliegen
        ;Welche davon man möchte, wird über das A32 Register übergeben, gibt es nur eine, muss es auf 0 gesetzt werden
        ;Eingabe Register
        ;   SRC64:          Absolute Quelle des Templates
        ;   DST64:          Absolutes Ziel
        ;   A64:            Template Nummer
        ;Ausgabe Register
        ;   DST64:  weitergezähltes Ziel
        ;Zerstörte Register
        ;   A,CNT,SRC
        
        ;Aus der Wahltabelle muss nun der Eintrag ausgewählt werden
        ;SRC64 zeigt auf den Tabellenanfang, die ersten 4 Bytes geben die Länge der Tabelle an, danach folgen die eigentlichen Einträge
        ;Jeder Eintrag ist 8 Byte lang. Die ersten 4 Byte geben den Offset zum Tabellenanfang, die nächsten 4 Byte die Länge des Codes
        mov CNT32,dword [SRC64+A64*8+8]
        mov A32,dword [SRC64+A64*8+4]
        add SRC64,A64
        ;Kopieren
        rep movsb
        ret
    .endOfFunction:

funActivateGeneratedFunction:
        ;Die Funktion wird vom Scratchpad an die eigentliche Stelle kopiert
        ;Eingabe Register:
        ;   HUB:            Basispointer zu den Hubregistern
        ;   SRC64:          Zeigt auf das 1. Byte nach der generierten Funktion
        ;                   Es wird dann von [hub_base+HubRelScratchpadOffset] .. SRC64-1 kopiert
        ;   DST64:          Absolutes Ziel
        ;Zerstörte Register:
        ;   CNT,SRC,DST
        
        ;Anzahl der zu kopierenden Bytes ermitteln
        lea CNT64, [SCRATCHPAD]
        sub CNT64, SRC64
        neg CNT64
        ;Startadresse erzeugen
        sub SRC64,CNT64
        ;kopieren
        rep movsb
        ret
    .endOfFunction:

funRunAsmSystemVAbi:
        ;Wird aus dem C++ Teil aufgerufen und sichert die notwendigen register
        ;1. Parameter: Absoluter Function Pointer
        ;2. Parameter: Hub Base
        ;3. Parameter: Cog Base oder null
        
        ;A64 braucht nicht gesichert zu werden
        push HUB
        push COG
        push B64
        push C64
        push CNT64
        push SRC64
        push DST64
        mov HUB, rsi
        mov COG, rdx
        call rdi
        pop DST64
        pop SRC64
        pop CNT64
        pop C64
        pop B64
        pop COG
        pop HUB
        ret
    .endOfFunction:
