hubRegs = {
    "name":"hub",
    "boolean" : ["ena_bus_reg","sys_c"],
    "uint32": [
            "acshr2","bus_ack_idx","bus_ashr2_wire","bus_d_wire","bus_q_reg","bus_q_reg_pre","bus_sel_idx","bus_state_wire",
            "cfg_reg","cnt_wire","cog_e","cog_ena_reg","dc","hub_state","lock_e","lock_state","pindir","pinout","pin_in_wire","sys_q"
    ],
    "uint64": [],
    "uint32array": [("hub_mem_ext",1024*24)]
}
    
cogRegs = {
    "name":"cog",
    "boolean":[
            "cflag",
            "reg_cog_ctra_delayed0","reg_cog_ctra_delayed1",
            "reg_cog_ctrb_delayed0","reg_cog_ctrb_delayed1",
            "reg_cog_vid_capr","reg_cog_vid_lastvidclk","reg_cog_vid_snc0","reg_cog_vid_snc1",
            "zflag"
    ],
    "uint32":[
            "bus_sel_state","compound_statex","dest","dira","instruction_reg","outa","p_reg","progctr",
            "reg_cog_ctra_ctramask","reg_cog_ctra_ctrbmask","reg_cog_ctra_ctrmode","reg_cog_ctra_ctrplldiv","reg_cog_ctra_frequency",
            "reg_cog_ctrb_ctramask","reg_cog_ctrb_ctrbmask","reg_cog_ctrb_ctrmode","reg_cog_ctrb_ctrplldiv","reg_cog_ctrb_frequency",
            "reg_cog_vid_baseband","reg_cog_vid_cnt0","reg_cog_vid_cnts","reg_cog_vid_composite","reg_cog_vid_discrete","reg_cog_vid_phase",
            "reg_cog_vid_saved_colors","reg_cog_vid_saved_pixels","reg_cog_vid_setr",
            "reg_cog_vid_vcfg_auralsub","reg_cog_vid_vcfg_chroma","reg_cog_vid_vcfg_vgroup","reg_cog_vid_vcfg_vmode","reg_cog_vid_vcfg_vpins",
            "reg_cog_vid_vscl_frameclocks","reg_cog_vid_vscl_pixelclocks","reg_pll_saved_both",
            "source"
    ],
    "uint64": ["reg_cog_ctra_phase","reg_cog_ctra_pllcounter","reg_cog_ctrb_phase","reg_cog_ctrb_pllcounter"],
    "uint32array": [("reg_cog_ram_r",512)]
}
    
def align(offset):
    while(offset%8!=0):
        offset+=1
    return offset

def generateRegCpp(indent,regs):
    subIndent = indent + "    "
    subSubIndent = subIndent+"    "
    subSubSubIndent = subSubIndent +"    "
    name = regs["name"]+"_regs"
    result = indent+"struct "+name+" {\n"
    serialize = subSubIndent+"stream"
    deserialize = subSubIndent+"stream"
    lstVars = ""
    constructorArgs = ["quint8* rawDataPointer"]
    constructor = ["basePtrOfThisStruct(rawDataPointer)"]
    offset = 0
    result += subIndent+"void* basePtrOfThisStruct;\n";
    for v in regs["uint64"]:
        if v!="":
            result += subIndent+"quint64& "+v+";\n"
            serialize += " << "+v;
            deserialize += " >> "+v;
            lstVars += "\n"+subSubSubIndent+"<< QPair<QString,QString>(\""+v+"\",\"0x\"+QString::number("+v+",16))"
            constructor.append(v+"(*(reinterpret_cast<quint64*>(&rawDataPointer["+str(offset)+"])))")
        offset += 8
    for v in regs["uint32"]:
        if v!="":
            result += subIndent+"quint32& "+v+";\n";
            serialize += " << "+v;
            deserialize += " >> "+v;
            lstVars += "\n"+subSubSubIndent+"<< QPair<QString,QString>(\""+v+"\",\"0x\"+QString::number("+v+",16))"
            constructor.append(v+"(*(reinterpret_cast<quint32*>(&rawDataPointer["+str(offset)+"])))")
        offset += 4
    for v in regs["boolean"]:
        if v!="":
            result += subIndent+"bool& "+v+";\n";
            serialize += " << "+v;
            deserialize += " >> "+v;
            lstVars += "\n"+subSubSubIndent+"<< QPair<QString,QString>(\""+v+"\","+v+" ? \"TRUE\" : \"false\")"
            constructor.append(v+"(*(reinterpret_cast<bool*>(&rawDataPointer["+str(offset)+"])))")
        offset += 1
    serialize += ";\n";
    deserialize += ";\n";
    for v in regs["uint32array"]:
        result += subIndent+"quint32* "+v[0]+";\n";
        serialize += subSubIndent+"for (int locIterator=0; locIterator<"+str(v[1])+"; locIterator++)\n"
        serialize += subSubIndent+"    stream << "+v[0]+"[locIterator];\n"
        deserialize += subSubIndent+"for (int locIterator=0; locIterator<"+str(v[1])+"; locIterator++)\n"
        deserialize += subSubIndent+"    stream >> "+v[0]+"[locIterator];\n"
        constructor.append(v[0]+"(reinterpret_cast<quint32*>(ptr_of_"+v[0]+"))")
        constructorArgs.append("void * ptr_of_"+v[0])
        offset += v[1]*4
    offset = align(offset)
    result += "\n\n";
    
    result += subIndent+name+"("+(",".join(constructorArgs))+"):"+(",".join(constructor))+"{}\n\n"
    
    result += subIndent+"void serialize(QDataStream& stream) const {\n"
    result += serialize;
    result += subIndent+"}\n\n"
    
    result += subIndent+"void deserialize(QDataStream& stream) {\n"
    result += deserialize;
    result += subIndent+"}\n\n"
    
    result += subIndent+"QList<QPair<QString,QString> > listInnerVars() const {\n"
    result += subSubIndent+"return QList<QPair<QString,QString> >()";
    result += lstVars+";\n";
    result += subIndent+"}\n\n"
    
    result += indent+"};\n\n"
    return result

def genCompleteCpp():
    f=open("propellerasmregsets.gen.h","w")
    f.write("#ifndef PROPELLERASMREGSETS_H\n")
    f.write("#define PROPELLERASMREGSETS_H\n\n")
    f.write("#include <QList>\n")
    f.write("#include <QString>\n")
    f.write("#include <QPair>\n")
    f.write("#include <QDataStream>\n\n")
    f.write("/////////////////////////////////////////\n")
    f.write("//WARNING: GENERATED FILE - DO NOT MODIFY\n")
    f.write("/////////////////////////////////////////\n\n")
    f.write("namespace PropellerAsmInternals {\n")
    f.write(generateRegCpp("    ",hubRegs))
    f.write(generateRegCpp("    ",cogRegs))
    f.write("};\n\n")
    f.write("#endif // PROPELLERASMREGSETS_H\n")
    f.close()

def generateRegAsm(indent,regs):
    result=""
    name = regs["name"]
    result += indent+";"+name+" Registers\n"
    offset = 0
    
    def genAsmReg(vname, tpe):
        return indent+(name+"_"+vname).ljust(40)+" equ "+str(offset).rjust(5)+" ;"+tpe+"\n"
    
    
    for v in regs["uint64"]:
        if v!="":
            result += genAsmReg(v,"quint64")
        offset += 8
    for v in regs["uint32"]:
        if v!="":
            result += genAsmReg(v,"quint32")
        offset += 4
    for v in regs["boolean"]:
        if v!="":
            result += genAsmReg(v,"bool")
        offset += 1
    offset = align(offset)
    result += indent+("total_size_of_"+name+"_regs").ljust(40)+" equ "+str(offset).rjust(5)+" ;sizeof("+name+"_regs)\n\n"
    
    result += "\n\n"
    
    return result

def genCompleteAsm():
    f=open("asm-regs.gen.asm","w")
    f.write(";/////////////////////////////////////////\n")
    f.write(";//WARNING: GENERATED FILE - DO NOT MODIFY\n")
    f.write(";/////////////////////////////////////////\n\n")
    f.write(generateRegAsm("",hubRegs))
    f.write(generateRegAsm("",cogRegs))
    f.close()
    
genCompleteCpp()
genCompleteAsm()

