#include "propellerasm.h"
#include "remainingpropellerasm.h"
#include "filetools.h"
#include <QtDebug>
#include <QElapsedTimer>
#include <sys/mman.h>

using namespace PropellerAsmInternals;

//TODO weg
extern "C" void cppMacroCog4SetControlRegisters(quint8* hubBasePtr, quint8 *cogBasePtr, quint32 instr_dst, quint32 outp_cog_alu_r) {
    hub_regs hubRegs(hubBasePtr,hubBasePtr+PropellerAsmInternals::MemoryMap::HubRelMemoryOffset);
    cog_regs cogRegs(cogBasePtr,cogBasePtr+PropellerAsmInternals::MemoryMap::CogRelMemoryOffset);
    //PropellerAsmImplGen::macroCog4SetControlRegisters(hubRegs,cogRegs, instr_dst, outp_cog_alu_r);
    customMacroCog4SetControlRegisters(hubRegs, cogRegs, instr_dst, outp_cog_alu_r);
}

struct PropellerAsmImpl {
    MemoryMap memoryMap;
    rs_hub hub;
    rs_cog cog0;
    rs_cog cog1;
    rs_cog cog2;
    rs_cog cog3;
    rs_cog cog4;
    rs_cog cog5;
    rs_cog cog6;
    rs_cog cog7;
    qint64 performanceNull;
    qint64 performancePre;
    qint64 performancePeripherals;
    qint64 performanceCog;
    qint64 performanceHub;
    qint64 performancePinOut;
    qint64 performancePllCycle;
    qint64 performanceCoarse;
    int performanceCounter;
    PropellerAsmImpl(QString sharedMemoryKey):
        memoryMap(sharedMemoryKey, true),
        hub(memoryMap),
        cog0(memoryMap.cogRegisters(0),memoryMap.cogMemory(0)),
        cog1(memoryMap.cogRegisters(1),memoryMap.cogMemory(1)),
        cog2(memoryMap.cogRegisters(2),memoryMap.cogMemory(2)),
        cog3(memoryMap.cogRegisters(3),memoryMap.cogMemory(3)),
        cog4(memoryMap.cogRegisters(4),memoryMap.cogMemory(4)),
        cog5(memoryMap.cogRegisters(5),memoryMap.cogMemory(5)),
        cog6(memoryMap.cogRegisters(6),memoryMap.cogMemory(6)),
        cog7(memoryMap.cogRegisters(7),memoryMap.cogMemory(7)),
        performanceNull(0),
        performancePre(0),
        performancePeripherals(0),
        performanceCog(0),
        performanceHub(0),
        performancePinOut(0),
        performancePllCycle(0),
        performanceCoarse(0),
        performanceCounter(0)
    {
        hub.cogs[0] = &cog0;
        hub.cogs[1] = &cog1;
        hub.cogs[2] = &cog2;
        hub.cogs[3] = &cog3;
        hub.cogs[4] = &cog4;
        hub.cogs[5] = &cog5;
        hub.cogs[6] = &cog6;
        hub.cogs[7] = &cog7;

        //daten auf 0 initialisieren
        memset(memoryMap.dataMemoryMap(),0x0,MemoryMap::TotalDataMemoryMapSize);
        //code auf nop (0x90) initialisieren
        memset(memoryMap.codeMemoryMap(),0x90,MemoryMap::TotalCodeMemoryMapSize);
        //scratchpad auf nop (0x90) initialisieren
        memset(memoryMap.scratchPad(),0x90,MemoryMap::ScratchPadSize);
        //code bereich ausführbar setzen
        mprotect(memoryMap.codeMemoryMap(), MemoryMap::TotalCodeMemoryMapSize,PROT_EXEC | PROT_READ | PROT_WRITE);
        rst();
    }
    ~PropellerAsmImpl() {
    }

    void serialize(QDataStream& stream) const {
        hub.serialize(stream);
        for (int i=0; i<8; ++i)
            hub.cogs[i]->serialize(stream);
    }

    void deserialize(QDataStream& stream) {
        hub.deserialize(stream);
        for (int i=0; i<8; ++i)
            hub.cogs[i]->deserialize(stream);
        updateAllDynamicFunctions();
    }

    void rst() {
        asmprop_hubRst(hub.basePtrOfThisStruct,nullptr);
        for (int i=0; i<8; ++i)
            asmprop_cogRst(hub.basePtrOfThisStruct,hub.cogs[i]->basePtrOfThisStruct);
        hub.ena_bus_reg = false;
        updateAllDynamicFunctions();
    }

    void updateAllDynamicFunctions() {
        for (int i=0; i<8; ++i)
            asmprop_cogUpdateAllDynamicFunctions(hub.basePtrOfThisStruct,hub.cogs[i]->basePtrOfThisStruct);
    }
#define MEASURE_PERFORMANCE
#define MEASURE_PERFORMANCE_FINE

    //////////////////////////////////////
    /// TODO Hier stehen geblieben
    /// In Cpp Version wurde PreEval und Pll vereinigt
    /// Dies auch in der ASM Version machen
    /// Hier gibt es einige Varianten zu beachten, wann pllsaved geupdatet werden muss
    ///
    /////////////////////////////////////////

    void evaluatePllClk(const quint32 pin_in) {
#ifdef MEASURE_PERFORMANCE_FINE
        QElapsedTimer elt;
        elt.start();
#endif
        hub.pin_in_wire = pin_in;
        asmprop_cogPreEvalPll(hub.basePtrOfThisStruct,nullptr);
#ifdef MEASURE_PERFORMANCE_FINE
        performancePllCycle+=elt.nsecsElapsed();
#endif
    }

    void evaluateCogClk(const quint32 pin_in, quint32& pin_out, quint32& pin_dir) {
#ifdef MEASURE_PERFORMANCE
        performanceCounter++;
        QElapsedTimer coarseElt;
        coarseElt.start();
#endif
#ifdef MEASURE_PERFORMANCE_FINE
        QElapsedTimer elt;
        elt.start();
        performanceNull+=elt.nsecsElapsed();
        elt.start();
#endif
        //Schritt 1: PLL Out von jedem Cog ermitteln (wird als Eingabewert für alle Cogs benötigt)
        hub.pin_in_wire = pin_in;
        asmprop_cogPreEvalClk(hub.basePtrOfThisStruct,nullptr);
        //Schritt 2: Alle Cogs Instruktionen und Counter/Videogenerator ausführen
        if (hub.ena_bus_reg)
            asmprop_cogBusSel(hub.basePtrOfThisStruct,nullptr);
#ifdef MEASURE_PERFORMANCE_FINE
        performancePre+=elt.nsecsElapsed();
        elt.start();
#endif
        asmprop_cogClkEna(hub.basePtrOfThisStruct,nullptr);
#ifdef MEASURE_PERFORMANCE_FINE
        performanceCog += elt.nsecsElapsed();
        elt.start();
#endif
        asmprop_cogClkCheckVid(hub.basePtrOfThisStruct,nullptr);
#ifdef MEASURE_PERFORMANCE_FINE
        performancePeripherals += elt.nsecsElapsed();
#endif
        //Schritt 3: Hub Transaktionen durchführen (jeden 2. Takt)
#ifdef MEASURE_PERFORMANCE_FINE
        elt.start();
#endif
        if (hub.ena_bus_reg) {
            asmprop_hubClkBusEna(hub.basePtrOfThisStruct,nullptr);
            hub.bus_sel_idx = (hub.bus_sel_idx+1)&7;
            PropellerAsmImplGen::hubClkPostEvalEna(hub,hub.bus_d_wire,hub.bus_ashr2_wire,hub.bus_state_wire);
            hub.ena_bus_reg = false;
        }
        else {
            PropellerAsmImplGen::hubClkBusNena(hub);
            hub.ena_bus_reg = true;
        }
        hub.cnt_wire++;
#ifdef MEASURE_PERFORMANCE_FINE
        performanceHub += elt.nsecsElapsed();
        elt.start();
#endif
        //Schritt 4 Ausgänge setzen
        asmprop_cogEvalPinOut(hub.basePtrOfThisStruct,nullptr);
        pin_out = hub.pinout;
        pin_dir = hub.pindir;
#ifdef MEASURE_PERFORMANCE_FINE
        performancePinOut += elt.nsecsElapsed();
#endif
#ifdef MEASURE_PERFORMANCE
        performanceCoarse += coarseElt.nsecsElapsed();
        if (performanceCounter>1000000) {
            qDebug()<<"PERF:"
                    <<"NUL"<<performanceNull/performanceCounter
                    <<"PRE"<<(performancePre-performanceNull)/performanceCounter
                    <<"PER"<<(performancePeripherals-performanceNull)/performanceCounter
                    <<"COG"<<(performanceCog-performanceNull)/performanceCounter
                    <<"HUB"<<(performanceHub-performanceNull)/performanceCounter
                    <<"OUT"<<(performancePinOut-performanceNull)/performanceCounter
                    <<"PLL"<<(performancePllCycle-performanceNull)/performanceCounter
                    <<"SUM"<<performanceCoarse/performanceCounter;
            performanceNull=0;
            performanceCounter=0;
            performancePre=0;
            performancePeripherals=0;
            performanceCog=0;
            performanceHub=0;
            performancePinOut=0;
            performancePllCycle=0;
            performanceCoarse=0;
        }
#endif
    }
};

PropellerAsm::PropellerAsm(QString deviceId, QString deviceName, QList<int> rom):
    AbstractPropeller(deviceId, deviceName),
    m_dig(new PropellerAsmImpl("asm-prop-mem-1")),
    m_pinIn(0),
    m_pinOut(0),
    m_pinDir(0)
{
    quint32* mem = m_dig->hub.hub_mem_ext;
    memset(mem,0,1024*96);
    if (rom.size() == 8192) {
        for (int j=0; j<8192; ++j) {
            auto v = static_cast<quint32>(rom[j]);
            mem[8192+j] = v;
            mem[16384+j] = PropellerAsmImplGen::decrypt(v);
        }
    }
    else
        qWarning()<<"Rom size mismatch"<<rom.size();
    reset();
}

PropellerAsm::~PropellerAsm() {
    delete m_dig;
}

void PropellerAsm::reset() {
    m_dig->rst();
    m_pinOut = 0;
    m_pinDir = 0;
}

void PropellerAsm::clockCycle() {
    m_dig->evaluatePllClk(m_pinIn);
    m_dig->evaluateCogClk(m_pinIn, m_pinOut, m_pinDir);
    AbstractPropeller::clockCycle();
    /*qDebug()<<"PropellerCloned::clockCycle() p4 snc0"<<m_dig->mod_cogs[4].mod_cog_vid.snc0
            <<"capr"<<m_dig->mod_cogs[4].mod_cog_vid.capr
            <<"snc1"<<m_dig->mod_cogs[4].mod_cog_vid.snc1
            <<"setr"<<m_dig->mod_cogs[4].mod_cog_vid.setr
            <<"frmclk"<<m_dig->mod_cogs[4].mod_cog_vid.vscl_frameclocks
            <<"pllafrq"<<m_dig->mod_cogs[4].mod_cog_ctra.frequency
            <<"pllacnt"<<m_dig->mod_cogs[4].mod_cog_ctra.pllcounter
            //<<"ctrmode"<<m_dig->mod_cogs[4].mod_cog_ctra.ctrmode
            <<"plladiv"<<m_dig->mod_cogs[4].mod_cog_ctra.ctrplldiv
            <<"plla"<<m_dig->mod_cogs[4].mod_cog_ctra.pll_saved
              ;*/
}

void PropellerAsm::setPinIn(quint32 pins) {
    m_pinIn = pins;
}

quint32 PropellerAsm::pinOut() const {
    return m_pinOut;
}

quint32 PropellerAsm::pinDir() const {
    return m_pinDir;
}

CogStatus PropellerAsm::cogStatus(int cogNo) const {
    auto& cog = *m_dig->hub.cogs[cogNo];
    CogStatus result;
    result.programCounter = int(cog.p_reg-1)&0x1FF;
    result.enabled = (m_dig->hub.cog_ena_reg & (1<<cogNo)) != 0;
    const quint32 decodedState = asmprop_decodeCogStatePointer(m_dig->hub.basePtrOfThisStruct,cog.basePtrOfThisStruct);
    bool decodedStateFound = decodedState != 0xFFFFFFFFU;
    result.running = decodedStateFound && ((decodedState & 0x00100000U) != 0U);
    result.carryFlag = cog.cflag;
    result.zeroFlag = cog.zflag;
    const quint32 tmpState = (decodedState>>16)&7;
    result.state = static_cast<CogStatus::State>((tmpState == 6 || !decodedStateFound) ? 0 : tmpState);
    return result;
}

quint32 PropellerAsm::getPinFromMask(quint32 mask) {
    for (quint32 i=0; i<32; ++i)
        if (mask == 1UL<<i)
            return i;
    return 0xFFFFFFFF;
}

CogCounterStatus PropellerAsm::cogCounterStatus(int cogNo, int counterNo) {
    CogCounterStatus result;
    if (counterNo != 0 && counterNo != 1)
        return result;
    const auto& cog = *m_dig->hub.cogs[cogNo];
    if (counterNo == 0) {
        result.phase = cog.reg_cog_ctra_phase;
        result.pllcounter = cog.reg_cog_ctra_pllcounter;
        result.frequency = cog.reg_cog_ctra_frequency;
        result.ctrmode = cog.reg_cog_ctra_ctrmode;
        result.ctrplldiv = cog.reg_cog_ctra_ctrplldiv;
        result.ctrapin = getPinFromMask(cog.reg_cog_ctra_ctramask);
        result.ctrbpin = getPinFromMask(cog.reg_cog_ctra_ctrbmask);
        result.delayed0 = cog.reg_cog_ctra_delayed0;
        result.delayed1 = cog.reg_cog_ctra_delayed1;
    }
    else {
        result.phase = cog.reg_cog_ctrb_phase;
        result.pllcounter = cog.reg_cog_ctrb_pllcounter;
        result.frequency = cog.reg_cog_ctrb_frequency;
        result.ctrmode = cog.reg_cog_ctrb_ctrmode;
        result.ctrplldiv = cog.reg_cog_ctrb_ctrplldiv;
        result.ctrapin = getPinFromMask(cog.reg_cog_ctrb_ctramask);
        result.ctrbpin = getPinFromMask(cog.reg_cog_ctrb_ctrbmask);
        result.delayed0 = cog.reg_cog_ctrb_delayed0;
        result.delayed1 = cog.reg_cog_ctrb_delayed1;
    }
    return result;
}

int PropellerAsm::cogMemory(int cogId, int address) const {
    return int(m_dig->hub.cogs[cogId]->reg_cog_ram_r[address]);
}

int PropellerAsm::hubMemory(int address) const {
    return int(m_dig->hub.hub_mem_ext[address]);
}

quint32 PropellerAsm::globalCounter() const {
    return m_dig->hub.cnt_wire;
}

void PropellerAsm::serialize(QDataStream& stream) const {
    AbstractPropeller::serialize(stream);
    m_dig->serialize(stream);
    stream << m_pinIn;
    stream << m_pinOut;
    stream << m_pinDir;
}

void PropellerAsm::deserialize(QDataStream& stream) {
    AbstractPropeller::deserialize(stream);
    m_dig->deserialize(stream);
    stream >> m_pinIn;
    stream >> m_pinOut;
    stream >> m_pinDir;
}

QList<QPair<QString,QString> > PropellerAsm::listInnerCogVariables(int cogNo) const {
    auto res=m_dig->hub.cogs[cogNo]->listInnerVars();
    const quint32 decodedState = asmprop_decodeCogStatePointer(m_dig->hub.basePtrOfThisStruct,m_dig->hub.cogs[cogNo]->basePtrOfThisStruct);
    res<<QPair<QString,QString>("compound_statex_decoded","0x"+QString::number(decodedState&0xFFFF,16));
    return res;
}

QList<QPair<QString,QString> > PropellerAsm::listInnerHubVariables() const {
    auto res=m_dig->hub.listInnerVars();
    const quint32 decodedState = asmprop_decodeHubStatePointer(m_dig->hub.basePtrOfThisStruct,nullptr);
    res<<QPair<QString,QString>("hub_state_decoded","0x"+QString::number(decodedState&0xFFFF,16));
    return res;
}

QString PropellerAsm::name() const {
    return "PropellerAsm";
}
