#ifndef PROPELLERASMPORTED_H
#define PROPELLERASMPORTED_H

#include "propellerasmregsets.gen.h"
#include "propellerasmmemorymap.h"
#include "propeller-asm/asm-callstubs.gen.h"

namespace PropellerAsmInternals {
    typedef cog_regs rs_cog;
    struct rs_hub : public hub_regs {
        rs_hub(MemoryMap &mm):hub_regs(mm.hubRegisters(),mm.hubMemory()),memoryMap(mm)
        {
            for (int i=0; i<8; ++i)
                cogs[i]=nullptr;
        }

        MemoryMap &memoryMap;
        rs_cog *cogs[8];
    };

    static void customMacroCog4SetControlRegisters (hub_regs& hubRegs,rs_cog& regcog,const quint32 instr_dst_input,const quint32 outp_cog_alu_r_input) {
        quint32 instr_dst = 0x0;
        quint32 outp_cog_alu_r = 0x0;
        instr_dst = instr_dst_input;
        outp_cog_alu_r = outp_cog_alu_r_input;
        if (instr_dst == 0x1f4) {
            regcog.outa = outp_cog_alu_r;
        }
        if (instr_dst == 0x1f6) {
            regcog.dira = outp_cog_alu_r;
        }
        if (instr_dst == 0x1ff) {
            regcog.reg_cog_vid_vscl_pixelclocks = ((outp_cog_alu_r >> 0xc) & 0xff);
            regcog.reg_cog_vid_vscl_frameclocks = (outp_cog_alu_r & 0xfff);
        }
        if (instr_dst == 0x1fe) {
            regcog.reg_cog_vid_vcfg_vmode = ((outp_cog_alu_r >> 0x1d) & 0x3);
            regcog.reg_cog_vid_vcfg_chroma = ((outp_cog_alu_r_input >> 0x1a) & 0x7);
            regcog.reg_cog_vid_vcfg_auralsub = ((outp_cog_alu_r >> 0x17) & 0x7);
            regcog.reg_cog_vid_vcfg_vgroup = (((outp_cog_alu_r >> 0x9) & 0x3) << 0x3);
            regcog.reg_cog_vid_vcfg_vpins = (outp_cog_alu_r & 0xff);
            //TODO nur updaten wenn auch notwendig
            asmprop_updateCogPeripheralsFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            asmprop_updateCogAsyncVid(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            asmprop_updateCogEvalPinOut(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
        }
        if (instr_dst == 0x1f9) {
            auto ctrModeNew = (outp_cog_alu_r >> 0x1a);
            auto pllDivNew = ((outp_cog_alu_r >> 0x17) & 0x7);
            if (ctrModeNew != regcog.reg_cog_ctrb_ctrmode) {
                regcog.reg_cog_ctrb_ctrmode = ctrModeNew;
                regcog.reg_cog_ctrb_ctrplldiv = pllDivNew;
                asmprop_updateCogPeripheralsFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
                asmprop_updateCogPreEvalFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
                asmprop_updateCogEvalPinOut(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            }
            else if (pllDivNew != regcog.reg_cog_ctrb_ctrplldiv) {
                regcog.reg_cog_ctrb_ctrplldiv = pllDivNew;
                asmprop_updateCogPreEvalFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            }
            regcog.reg_cog_ctrb_ctrbmask = (0x1 << ((outp_cog_alu_r >> 0x9) & 0x1f));
            regcog.reg_cog_ctrb_ctramask = (0x1 << (outp_cog_alu_r & 0x1f));
        }
        if (instr_dst == 0x1fb) {
            regcog.reg_cog_ctrb_frequency = outp_cog_alu_r;
        }
        if (instr_dst == 0x1fd) {
            regcog.reg_cog_ctrb_phase = quint64(outp_cog_alu_r);
        }
        if (instr_dst == 0x1f8) {
            auto ctrModeNew = (outp_cog_alu_r >> 0x1a);
            auto pllDivNew = ((outp_cog_alu_r >> 0x17) & 0x7);
            if (ctrModeNew != regcog.reg_cog_ctra_ctrmode) {
                regcog.reg_cog_ctra_ctrmode = ctrModeNew;
                regcog.reg_cog_ctra_ctrplldiv = pllDivNew;
                asmprop_updateCogPeripheralsFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
                asmprop_updateCogPreEvalFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
                asmprop_updateCogEvalPinOut(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            }
            else if (pllDivNew != regcog.reg_cog_ctra_ctrplldiv) {
                regcog.reg_cog_ctra_ctrplldiv = pllDivNew;
                asmprop_updateCogPreEvalFunction(hubRegs.basePtrOfThisStruct,regcog.basePtrOfThisStruct);
            }
            regcog.reg_cog_ctra_ctrbmask = (0x1 << ((outp_cog_alu_r >> 0x9) & 0x1f));
            regcog.reg_cog_ctra_ctramask = (0x1 << (outp_cog_alu_r & 0x1f));
        }
        if (instr_dst == 0x1fa) {
            regcog.reg_cog_ctra_frequency = outp_cog_alu_r;
        }
        if (instr_dst == 0x1fc) {
            regcog.reg_cog_ctra_phase = quint64(outp_cog_alu_r);
        }
    }
}

#endif // PROPELLERASMPORTED_H
