#include "propellernewcpp.h"
#include "propeller-newcpp/generatedpropellernew.gen.h"
#include <QtDebug>

constexpr quint32 PropellerNewCppImplGen::lookuptable0[];
constexpr quint32 PropellerNewCppImplGen::lookuptable1[];
constexpr quint32 PropellerNewCppImplGen::lookuptable2[];
constexpr quint32 PropellerNewCppImplGen::lookuptable3[];
constexpr quint32 PropellerNewCppImplGen::lookuptable4[];
constexpr quint32 PropellerNewCppImplGen::lookuptable5[];

struct PropellerNewCppImpl {

    PropellerNewCppImplGen::rs_hub hub;
    PropellerNewCppImplGen::rs_cog cogs[8];
    quint32 statsCounter;
    PropellerNewCppImpl():statsCounter(0) {
        for (int i=0; i<8; ++i) {
            cogs[i].cogId=quint32(i);
            hub.cogs[i] = &cogs[i];
        }
    }

    void serialize(QDataStream& stream) const {
        hub.serialize(stream);
        for (int i=0; i<8; ++i)
            cogs[i].serialize(stream);
    }

    void deserialize(QDataStream& stream) {
        hub.deserialize(stream);
        for (int i=0; i<8; ++i)
            cogs[i].deserialize(stream);
    }

    void rst() {
        PropellerNewCppImplGen::hubRst(hub);
        for (int i=0; i<8; ++i)
            PropellerNewCppImplGen::cogRst(hub,cogs[i]);
        hub.ena_bus_reg = false;
    }

    void evaluatePllClk(const quint32 pin_in) {
        hub.pin_in_wire = pin_in;
        for (int i=0; i<8; ++i) {
            quint32 pll_saved_both=0;
            PropellerNewCppImplGen::cogEvalPreAndPll(hub,cogs[i],pll_saved_both);
            //plla_saved kann sich während clkpll ändern
            if (cogs[i].reg_cog_vid_vcfg_vmode != 0) {
                const bool clkvid = (pll_saved_both&1)==1;
                if (clkvid && !cogs[i].reg_cog_vid_lastvidclk) {
                    PropellerNewCppImplGen::cogEvalMain_async_vclk(hub,cogs[i]);
                }
                cogs[i].reg_cog_vid_lastvidclk = clkvid;
            }
        }
    }

    bool pllIsIdle(quint32 mode) {
        return mode == 0x0 || mode == 0x4 || mode == 0x8 || mode == 0xc || mode == 0x10 || mode == 0x14 || mode == 0x18 || mode >= 0x1c;
    }

    void evaluateCogClk(const quint32 pin_in, quint32& pin_out, quint32& pin_dir) {
        if (statsCounter>100000) {
            statsCounter=0;
            /*QString pllDbgStr;
            for (int i=0; i<8; ++i)
                pllDbgStr += " "+QString(pllIsIdle(cogs[i].reg_cog_ctra_ctrmode) ? "_" : "A")+QString(pllIsIdle(cogs[i].reg_cog_ctrb_ctrmode) ? "_" : "B");
            qDebug()<<"plldbg"<<pllDbgStr;*/
        }
        else
            statsCounter++;
        //Schritt 1: PLL Out von jedem Cog ermitteln (wird als Eingabewert für alle Cogs benötigt)
        hub.pin_in_wire = pin_in;
        for (int i=0; i<8; ++i) {
            cogs[i].reg_pll_saved_both=0;
            PropellerNewCppImplGen::cogEvalPreAndPll(hub,cogs[i],cogs[i].reg_pll_saved_both);
        }
        //Schritt 2: Alle Cogs Instruktionen und Counter/Videogenerator ausführen
        if (hub.ena_bus_reg)
            PropellerNewCppImplGen::cogClkBusSelDispatcher(hub,cogs[hub.bus_sel_idx]);
        for (int i=0; i<8; ++i)
            PropellerNewCppImplGen::cogClkDispatchCompoundState(hub, cogs[i]);
        for (int i=0; i<8; ++i) {
            //plla_saved kann sich während clkpll ändern
            if (cogs[i].reg_cog_vid_vcfg_vmode != 0) {
                const bool clkvid = (cogs[i].reg_pll_saved_both&1)==1;
                if (clkvid && !cogs[i].reg_cog_vid_lastvidclk) {
                    PropellerNewCppImplGen::cogEvalMain_async_vclk(hub,cogs[i]);
                }
                cogs[i].reg_cog_vid_lastvidclk = clkvid;
            }
        }
        //Schritt 3: Hub Transaktionen durchführen (jeden 2. Takt)
        if (hub.ena_bus_reg) {
            const quint32 oldCogEna=hub.cog_ena_reg;
            PropellerNewCppImplGen::hubClkEnaBusDispatcher(hub);
            hub.bus_sel_idx = (hub.bus_sel_idx+1)&7;
            PropellerNewCppImplGen::hubClkPostEvalEna(hub,hub.bus_d_wire,hub.bus_ashr2_wire,hub.bus_state_wire);
            hub.ena_bus_reg = false;

            //abgeleitete clocks untersuchen
            if (hub.cog_ena_reg != oldCogEna) {
                for (int i=0; i<8; ++i) {
                    //ena kann sich nur nach clkcog ändern
                    const bool ena = (hub.cog_ena_reg & (1U<<i)) != 0U;
                    const bool lastEna = (oldCogEna & (1U<<i)) != 0U;
                    if (!ena && lastEna)
                        PropellerNewCppImplGen::cogNegedgeEna(hub,cogs[i]);
                    else if (ena && !lastEna)
                        PropellerNewCppImplGen::cogPosedgeEna(hub,cogs[i]);
                }
            }
        }
        else {
            PropellerNewCppImplGen::hubClkBusNena(hub);
            hub.ena_bus_reg = true;
        }
        hub.cnt_wire++;

        //Schritt 4 Ausgänge setzen
        hub.pinout = 0;
        hub.pindir = 0;
        for (int i=0; i<8; ++i) {
            quint32 cogPinOut=0;
            quint32 cogPinDir=0;
            PropellerNewCppImplGen::cogEvalMain_pinout(hub,cogs[i],cogPinOut,cogPinDir);
            hub.pinout |= cogPinOut;
            hub.pindir |= cogPinDir;
        }
        pin_out = hub.pinout;
        pin_dir = hub.pindir;
    }
};

PropellerNewCpp::PropellerNewCpp(QString deviceId, QString deviceName, QList<int> rom):
    AbstractPropeller(deviceId, deviceName),
    m_dig(new PropellerNewCppImpl()),
    m_pinIn(0),
    m_pinOut(0),
    m_pinDir(0)
{
    quint32* mem = m_dig->hub.hub_mem_ext;
    memset(mem,0,1024*96);
    if (rom.size() == 8192) {
        for (int j=0; j<8192; ++j) {
            auto v = static_cast<quint32>(rom[j]);
            mem[8192+j] = v;
            mem[16384+j] = PropellerNewCppImplGen::decrypt(v);
        }
    }
    else
        qWarning()<<"Rom size mismatch"<<rom.size();
    reset();
}

PropellerNewCpp::~PropellerNewCpp() {
    delete m_dig;
}

void PropellerNewCpp::reset() {
    m_dig->rst();
    m_pinOut = 0;
    m_pinDir = 0;
}

void PropellerNewCpp::clockCycle() {
    m_dig->evaluatePllClk(m_pinIn);
    m_dig->evaluateCogClk(m_pinIn, m_pinOut, m_pinDir);
    AbstractPropeller::clockCycle();
}

void PropellerNewCpp::setPinIn(quint32 pins) {
    m_pinIn = pins;
}

quint32 PropellerNewCpp::pinOut() const {
    return m_pinOut;
}

quint32 PropellerNewCpp::pinDir() const {
    return m_pinDir;
}

CogStatus PropellerNewCpp::cogStatus(int cogNo) const {
    auto& cog = m_dig->cogs[cogNo];
    CogStatus result;
    result.programCounter = int(cog.p_reg-1)&0x1FF;
    result.enabled = (m_dig->hub.cog_ena_reg & (1<<cogNo)) != 0;
    result.running = PropellerNewCppImplGen::compoundStateToRunning(cog);
    result.carryFlag = cog.cflag;
    result.zeroFlag = cog.zflag;
    const quint32 tmpState = PropellerNewCppImplGen::compoundStateToState(cog);
    result.state = static_cast<CogStatus::State>(tmpState == 6 ? 0 : tmpState);
    return result;
}

quint32 PropellerNewCpp::getPinFromMask(quint32 mask) {
    for (quint32 i=0; i<32; ++i)
        if (mask == 1UL<<i)
            return i;
    return 0xFFFFFFFF;
}

CogCounterStatus PropellerNewCpp::cogCounterStatus(int cogNo, int counterNo) {
    CogCounterStatus result;
    if (counterNo != 0 && counterNo != 1)
        return result;
    const auto& cog = m_dig->cogs[cogNo];
    if (counterNo == 0) {
        result.phase = cog.reg_cog_ctra_phase;
        result.pllcounter = cog.reg_cog_ctra_pllcounter;
        result.frequency = cog.reg_cog_ctra_frequency;
        result.ctrmode = cog.reg_cog_ctra_ctrmode;
        result.ctrplldiv = cog.reg_cog_ctra_ctrplldiv;
        result.ctrapin = getPinFromMask(cog.reg_cog_ctra_ctramask);
        result.ctrbpin = getPinFromMask(cog.reg_cog_ctra_ctrbmask);
        result.delayed0 = cog.reg_cog_ctra_delayed0;
        result.delayed1 = cog.reg_cog_ctra_delayed1;
    }
    else {
        result.phase = cog.reg_cog_ctrb_phase;
        result.pllcounter = cog.reg_cog_ctrb_pllcounter;
        result.frequency = cog.reg_cog_ctrb_frequency;
        result.ctrmode = cog.reg_cog_ctrb_ctrmode;
        result.ctrplldiv = cog.reg_cog_ctrb_ctrplldiv;
        result.ctrapin = getPinFromMask(cog.reg_cog_ctrb_ctramask);
        result.ctrbpin = getPinFromMask(cog.reg_cog_ctrb_ctrbmask);
        result.delayed0 = cog.reg_cog_ctrb_delayed0;
        result.delayed1 = cog.reg_cog_ctrb_delayed1;
    }
    return result;
}

int PropellerNewCpp::cogMemory(int cogId, int address) const {
    return int(m_dig->cogs[cogId].reg_cog_ram_r[address]);
}

int PropellerNewCpp::hubMemory(int address) const {
    return int(m_dig->hub.hub_mem_ext[address]);
}

quint32 PropellerNewCpp::globalCounter() const {
    return m_dig->hub.cnt_wire;
}

void PropellerNewCpp::serialize(QDataStream& stream) const {
    AbstractPropeller::serialize(stream);
    m_dig->serialize(stream);
    stream << m_pinIn;
    stream << m_pinOut;
    stream << m_pinDir;
}

void PropellerNewCpp::deserialize(QDataStream& stream) {
    AbstractPropeller::deserialize(stream);
    m_dig->deserialize(stream);
    stream >> m_pinIn;
    stream >> m_pinOut;
    stream >> m_pinDir;
}

QList<QPair<QString,QString> > PropellerNewCpp::listInnerCogVariables(int cogNo) const {
    return m_dig->cogs[cogNo].listInnerVars();
}

QList<QPair<QString,QString> > PropellerNewCpp::listInnerHubVariables() const {
    return m_dig->hub.listInnerVars();
}

QString PropellerNewCpp::name() const {
    return "PropellerNewCpp";
}
