#include "propellerverilator.h"
#include "Vdig.h"
#include <verilated.h>
#include <verilated.cpp>
#include <QtDebug>
#include "cogalu.h"

extern VerilatorCogStatus allCogs[256];
extern int allHubMemory[32][16384];

VerilatorCogStatus allCogs[256];
int allHubMemory[32][16384];

extern "C" void cog_set_ram(int globalCogId, int address, int data) {
    allCogs[globalCogId&0xFF].cogram[address] = data;
}

extern "C" void cog_set_pc(int globalCogId, int pc, int flags) {
    allCogs[globalCogId&0xFF].programCounter = pc;
    allCogs[globalCogId&0xFF].flags = flags;
}

extern "C" void hub_write_mem(int globalHubId, int address, int data, int mask) {
    //qDebug()<<"hub_write_mem"<<address<<data<<mask;
    allHubMemory[globalHubId][address] = (data&(~mask)) | (allHubMemory[globalHubId][address]&(mask));
    //qDebug()<<"HUBMEM"<<address<<QString::number(quint32(allHubMemory[globalHubId][address]),16).rightJustified(8,'0');
}

extern "C" int hub_read_mem(int globalHubId, int address) {
    //qDebug()<<"RD"<<address<<allHubMemory[globalHubId][address];
    return allHubMemory[globalHubId][address];
}

//extern "C" void cog_evaluate_alu(quint32 i, quint32 s, quint32 d, quint32 p, quint32 run, quint32 ci, quint32 zi, quint32 bus_q, quint32 bus_c, quint8 *wr, quint32 *r, quint8 *co, quint8 *zo) {
//extern "C" void cog_evaluate_alu(quint32 i, quint32 s, quint32 d, quint32 p, quint32 run, quint32 ci, quint32 zi, quint32 bus_q, quint32 bus_c, quint8 &wr, quint32 &r, quint8 &co, quint8 &zo) {
extern "C" void cog_evaluate_alu(int i, int s, int d, int p, int run, int ci, int zi, int bus_q, int bus_c, quint8 *wr, int *r, quint8 *co, quint8 *zo) {
    CogAluResult res((PropellerInstruction)i,s,d,p,run,ci,zi,bus_q,bus_c);
    //qDebug()<<"cog_evaluate_alu"<<i<<s<<res.result;
    *wr = res.writeEnable;
    *r = res.result;
    *co = res.carryOut;
    *zo = res.zeroOut;
}

struct PropellerVerilatorImpl {
    PropellerVerilatorImpl():top(nullptr) {}
    Vdig *top;
};

PropellerVerilator::PropellerVerilator(QString deviceId, QString deviceName, int hubId, QList<int> rom):
    AbstractPropeller(deviceId, deviceName),
    impl(new PropellerVerilatorImpl()),
    m_hubId(hubId)
{
    if (rom.size() == 8192) {
        for (int j=0; j<8192; ++j)
            allHubMemory[hubId][8192+j] = static_cast<int>(rom[j]);
    }
    else
        qWarning()<<"Rom size mismatch"<<rom.size();

    impl->top = new Vdig;
    impl->top->debugGlobalHubId = m_hubId;
    reset();
}

void PropellerVerilator::reset() {
    impl->top->nres = 1;
    impl->top->eval();
    impl->top->nres = 0;
    impl->top->eval();
    clockCycle();
    clockCycle();
    impl->top->nres = 1;
    impl->top->eval();
}

void PropellerVerilator::clockCycle() {
    for (int k=0; k<4; ++k) {
        impl->top->clk_cog=(k&2)>>1;
        impl->top->clk_pll=k&1;
        impl->top->eval();
    }
    AbstractPropeller::clockCycle();
}

void PropellerVerilator::setPinIn(quint32 pins) {
    impl->top->pin_in = pins;
}

quint32 PropellerVerilator::pinOut() const {
    return impl->top->pin_out;
}

quint32 PropellerVerilator::pinDir() const {
    return impl->top->pin_dir;
}

CogStatus PropellerVerilator::cogStatus(int cogNo) const {
    CogStatus result;
    const VerilatorCogStatus &cog(allCogs[m_hubId*8+cogNo]);
    result.programCounter = cog.programCounter;
    result.running = (cog.flags & (VerilatorCogStatus::FLAG_ENABLED | VerilatorCogStatus::FLAG_RUNNING)) == (VerilatorCogStatus::FLAG_ENABLED | VerilatorCogStatus::FLAG_RUNNING);
    result.enabled = (cog.flags & VerilatorCogStatus::FLAG_ENABLED) !=0;
    result.zeroFlag = (cog.flags & VerilatorCogStatus::FLAG_ZERO) != 0;
    result.carryFlag = (cog.flags & VerilatorCogStatus::FLAG_CARRY) != 0;
    //result.stateCan = (cog.flags & VerilatorCogStatus::FLAG_STATE_CANCEL) != 0;

    const bool state_read_s = (cog.flags & VerilatorCogStatus::FLAG_STATE_RDS) != 0;
    const bool state_read_d = (cog.flags & VerilatorCogStatus::FLAG_STATE_RDD) != 0;
    const bool state_read_ni = (cog.flags & VerilatorCogStatus::FLAG_STATE_RDI) != 0;
    const bool state_write_d = (cog.flags & VerilatorCogStatus::FLAG_STATE_WRD) != 0;
    const bool state_wait = (cog.flags & VerilatorCogStatus::FLAG_STATE_WAIT) != 0;

    if (!state_read_s && !state_read_d && !state_read_ni && !state_write_d && !state_wait)
        result.state = CogStatus::StateNone;
    else if (state_read_s && !state_read_d && !state_read_ni && !state_write_d && !state_wait)
        result.state = CogStatus::StateRdS;
    else if (!state_read_s && state_read_d && !state_read_ni && !state_write_d && !state_wait)
        result.state = CogStatus::StateRdD;
    else if (!state_read_s && !state_read_d && state_read_ni && !state_write_d && !state_wait)
        result.state = CogStatus::StateRdI;
    else if (!state_read_s && !state_read_d && !state_read_ni && state_write_d && !state_wait)
        result.state = CogStatus::StateWrD;
    else if (!state_read_s && !state_read_d && !state_read_ni && !state_write_d && state_wait)
        result.state = CogStatus::StateWai;
    else
        result.state = CogStatus::StateIllg;
    return result;
}

CogCounterStatus PropellerVerilator::cogCounterStatus(int, int) {
    throw HWSerializeException("PropellerVerilator::cogCounterStatus not implemented");
}

int PropellerVerilator::cogMemory(int cogId, int address) const {
    return allCogs[m_hubId*8+cogId].cogram[address];
}

int PropellerVerilator::hubMemory(int address) const {
    return hub_read_mem(m_hubId,address);
}

quint32 PropellerVerilator::globalCounter() const {
    return 0; //TODO
}

void PropellerVerilator::serialize(QDataStream& stream) const {
    AbstractPropeller::serialize(stream);
    throw HWSerializeException("PropellerVerilator does not support serialize");
}

void PropellerVerilator::deserialize(QDataStream& stream) {
    AbstractPropeller::deserialize(stream);
    throw HWSerializeException("PropellerVerilator does not support deserialize");
}

QList<QPair<QString,QString> > PropellerVerilator::listInnerCogVariables(int) const {
    return QList<QPair<QString,QString> >(); //wird nicht unterstützt
}

QList<QPair<QString,QString> > PropellerVerilator::listInnerHubVariables() const {
    return QList<QPair<QString,QString> >(); //wird nicht unterstützt
}

QString PropellerVerilator::name() const {
    return "PropellerVerilator";
}
