#ifndef PROPELLERVERILATOR_H
#define PROPELLERVERILATOR_H

#include <QString>
#include "abstractpropeller.h"

struct VerilatorCogStatus {
    VerilatorCogStatus():programCounter(0),flags(0) {
        for (int i=0; i<512; ++i)
            cogram[i]=0;
    }
    enum Flags {
        FLAG_ENABLED=1<<0,
        FLAG_RUNNING=1<<1,
        FLAG_STATE_RDS=1<<2,
        FLAG_STATE_RDD=1<<3,
        FLAG_STATE_RDI=1<<4,
        FLAG_STATE_WRD=1<<5,
        FLAG_STATE_WAIT=1<<6,
        FLAG_STATE_CANCEL=1<<7,
        FLAG_CARRY=1<<8,
        FLAG_ZERO=1<<9,

        STATE_MASK = FLAG_STATE_RDS | FLAG_STATE_RDD | FLAG_STATE_RDI | FLAG_STATE_WRD | FLAG_STATE_WAIT | FLAG_STATE_CANCEL};
    int cogram[512];
    int programCounter;
    int flags;
};

struct PropellerVerilatorImpl;

class PropellerVerilator : public AbstractPropeller {
public:
    PropellerVerilator(QString deviceId, QString deviceName, int hubId, QList<int> rom);
    virtual void reset();
    virtual void clockCycle();

    virtual void setPinIn(quint32 pins);
    virtual quint32 pinOut() const;
    virtual quint32 pinDir() const;

    virtual CogStatus cogStatus(int cogNo) const;
    virtual CogCounterStatus cogCounterStatus(int cogNo, int counterNo);
    virtual int cogMemory(int cogId, int address) const;
    virtual int hubMemory(int address) const;
    virtual quint32 globalCounter() const;

    virtual void serialize(QDataStream& stream) const;
    virtual void deserialize(QDataStream& stream);

    virtual QList<QPair<QString,QString> > listInnerCogVariables(int cogNo) const;
    virtual QList<QPair<QString,QString> > listInnerHubVariables() const;

    virtual QString name() const;
private:
    PropellerVerilatorImpl *impl;
    int m_hubId;
};

#endif // PROPELLERVERILATOR_H
