#include "propellerwidget.h"
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>

PropellerWidget::PropellerWidget():
    m_propeller(nullptr),
    m_name(nullptr),
    m_spbCog(nullptr),
    m_splitter(nullptr),
    m_cogViewer(nullptr),
    m_hubViewer(nullptr)
{
    auto mainLayout = new QVBoxLayout(this);
    auto layoutTop = new QHBoxLayout();
    mainLayout->addLayout(layoutTop);
    layoutTop->addWidget(new QLabel(tr("Cog No"),this));
    m_spbCog = new QSpinBox(this);
    m_name = new QLabel(this);
    layoutTop->addWidget(m_name);
    layoutTop->addWidget(m_spbCog);
    m_spbCog->setMaximum(7);
    m_splitter = new QSplitter(this);
    mainLayout->addWidget(m_splitter);

    m_cogViewer = new CogViewer(m_splitter);
    m_splitter->addWidget(m_cogViewer);

    m_hubViewer = new HubViewer(m_splitter);
    m_splitter->addWidget(m_hubViewer);

    connect(m_spbCog,SIGNAL(valueChanged(int)),this,SLOT(cogNoChanged(int)));
    setPropeller(nullptr);
}

void PropellerWidget::cogNoChanged(int) {
    if (m_propeller)
        m_cogViewer->setPropeller(m_propeller, m_spbCog->value());
}

void PropellerWidget::setPropeller(AbstractPropeller* propeller) {
    m_propeller = propeller;
    m_cogViewer->setPropeller(m_propeller,m_spbCog->value());
    m_hubViewer->setPropeller(propeller);
    m_name->setText(m_propeller ? m_propeller->name() : tr("No Propeller assigned"));
    setWindowTitle(m_name->text());
}

void PropellerWidget::updateViews() {
    m_cogViewer->updateState(true);
    m_hubViewer->updateState();
}
