#ifndef PS2DEVICE_H
#define PS2DEVICE_H

#include "hwtrace.h"
#include <QMap>

class PS2Device
{
public:
    PS2Device(quint32 clockPin, quint32 dataPin);
    virtual ~PS2Device();
    quint32 clockCycle(quint32 pulledUpPins); //ausgabe ist wired and, d.h. eine 0 zieht des bus nach unten
protected:
    virtual QByteArray byteReceived(int byte)=0;
    void transmitPackage(QByteArray pkg); //not for command responses
private:
    bool hasOddBits(int data) const;
    void transmitByte(int b);
    void removeFinishedPackages();
    enum State {Idle,HostWaitStart,HostToDeviceClkLow,HostToDeviceClkHigh,HostToDeviceStop1,HostToDeviceStop2,HostToDeviceStop3,DeviceToHostClockLow,DeviceToHostClockHigh};
    QByteArray m_commandResponse;
    quint32 m_clockMask;
    quint32 m_dataMask;
    quint32 m_remainingMask;
    quint32 m_clockDivider;
    quint32 m_pinOut;
    int m_bitCounter;
    int m_recv;
    int m_send;
    int m_packageSendPos;
    State m_state;
    QList<QByteArray> m_packages;
};

class PS2Keyboard : public PS2Device {
public:
    PS2Keyboard(quint32 clockPin, quint32 dataPin);
    void keyAction(int key, bool press);
protected:
    virtual QByteArray byteReceived(int byte);
private:
    struct ScanCode {
        ScanCode():c1(-1) {}
        ScanCode(int c1):c1(c1) {}
        int c1;
    };
    enum KeyboardState {KbdIdle,KbdSetTypematicRateDelay,KbdSetLeds};
    KeyboardState m_kbdState;
    QMap<int,ScanCode> m_keyMap;
};

#endif // PS2DEVICE_H
