#include "simulationmaster.h"
#include "propeller-gencpp/propellergencpp.h"
#include "propeller-newcpp/propellernewcpp.h"
#include "propeller-verilator/propellerverilator.h"
#include "propeller-asm/propellerasm.h"
#include "filetools.h"
#include <QtDebug>

SimulationMaster::SimulationMaster(PropellerImplementation impl):
    m_propeller(propellerFactory(impl,"M.P","Propeller")),
    m_bootEEPromI2CEndpoint(new I2CEndpoint("M.I","Boot I2C Endpoint",28,29)),
    m_bootEEProm(new I2CEEPRom("M.I.E","Boot EEPROM",
                               //FileTools::readBinary("../hive-software/tutorial-testprogramme/schritt-3-bellatrix/test-3-1-bellatrix-blinky/start-blinky.eeprom")
                               //FileTools::readBinary("../hive-software/tutorial-testprogramme/schritt-4-bellatrix-vga/test-4-1-bellatrix-vga-text-hallo/start_vga_hallo.eeprom")
                               //FileTools::readBinary("../hive-software/tutorial-testprogramme/schritt-5-bellatrix-tv-keyb-maus/test-5-1-bellatrix-vga-tastatur/start_vga_tastatur.eeprom")
                               //FileTools::readBinary("../hive-software/tutorial-testprogramme/schritt-5-bellatrix-tv-keyb-maus/test-5-8-bellatrix-matrix/start_tv_matrix.eeprom")
                               //FileTools::readBinary("../hive-software/tutorial-testprogramme/turbulence.eeprom")
                               FileTools::readBinary("other-files/turbulence.eeprom")

                     ,64*1024)),
    //m_vgaDecoder(nullptr),
    //m_vgaDecoder(new VGADecoder("M.V","VGA Screen",8)), //matrix
    //m_keyboard(new PS2Keyboard(16,17)),
    m_vgaDecoder(new VGADecoder("M.V","VGA Screen",16)), //turbulence
    m_keyboard(nullptr),
    m_lastVCDPins(0),
    m_lowSpeedSkipCounter(0),
    m_lastSDOPins(0)
{
    m_bootEEPromI2CEndpoint->addDevice(m_bootEEProm);
    if (m_vgaDecoder != nullptr)
        connect(m_vgaDecoder,SIGNAL(nextFrame(QImage)),this,SIGNAL(nextFrame(QImage)));

    /*m_vcdLogFile.setFileName("other-files/pins.vcd");
    m_vcdLogFile.open(QIODevice::WriteOnly);
    m_vcdLogFile.write("$timescale 1ps $end\n\
                       $scope module logic $end\n\
                       $var wire 1 c scl $end\n\
                       $var wire 1 d sda $end\n\
                       $var wire 1 o sdo $end\n\
                       $upscope $end\n\
                       $enddefinitions $end\n\
                       $dumpvars\n\
                       0c\n\
                       0d\n\
                       0o\n");*/

}

AbstractPropeller* SimulationMaster::propellerFactory(PropellerImplementation impl, QString deviceId, QString deviceName) {
    auto rom = FileTools::readIntelHex32("other-files/hub_rom_low.hex")+FileTools::readIntelHex32("other-files/hub_rom_high.hex");
    switch(impl) {
        case PropGenCpp: return new PropellerGenCpp(deviceId, deviceName, rom);
        case PropNewCpp: return new PropellerNewCpp(deviceId, deviceName, rom);
        case PropVerilator: throw QString("not implemented"); //return new PropellerVerilator(deviceId, deviceName, 0, rom);
        case PropAsm: return new PropellerAsm(deviceId, deviceName, rom);
    }
    return nullptr;
}

void SimulationMaster::clockCycle() {
    m_propeller->clockCycle();
    //m_bootEEProm->setPins(m_propeller->pinIsTriState(28) || m_propeller->pinOutValue(28), m_propeller->pinIsTriState(29) || m_propeller->pinOutValue(29));

    quint32 pulledUpPins = ~(~m_propeller->pinOut() & m_propeller->pinDir());

    pulledUpPins&=0x3FFFFFFF; //TODO deaktiviert uart zum schnelleren booten (pins 31/30)

    if (m_vgaDecoder)
        m_vgaDecoder->clockCycle(pulledUpPins);

    const quint32 inPins = pulledUpPins;
    //m_lowSpeedSkipCounter++;
    //if (m_lowSpeedSkipCounter>10) {
        m_lastSDOPins = 0xFFFFFFFF;
        if (m_keyboard)
            m_lastSDOPins &= m_keyboard->clockCycle(pulledUpPins);
        const quint32 sdo = m_bootEEPromI2CEndpoint->clockCycle(pulledUpPins);
        m_lastSDOPins &= sdo;
        m_lowSpeedSkipCounter=0;
    //}


#if 0
    quint32 vcdPins = ((pulledUpPins>>28)&3) | (((sdo>>29)&1)<<2);
    if (vcdPins != m_lastVCDPins) {
        m_lastVCDPins = vcdPins;
        //qDebug()<<"VCD PINS CHANGED"<<m_propeller->globalCounter()<<vcdPins;
        //m_vcdLogFile.write(("#"+QString::number(m_propeller->globalCounter())+"\n"+QString::number((vcdPins)&1)+"c\n"+QString::number((vcdPins>>1)&1)+"d\n"+QString::number((vcdPins>>2)&1)+"o\n").toLatin1());
        //m_vcdLogFile.flush();
    }
#endif

    m_propeller->setPinIn(inPins & m_lastSDOPins);
}

QList<HWTraceable*> SimulationMaster::getDevices() {
    QList<HWTraceable*> result;
    result << m_propeller<<m_bootEEPromI2CEndpoint<<m_bootEEProm;
    if (m_vgaDecoder)
        result <<m_vgaDecoder;
     //TODO ps2 devices
    return result;
}

AbstractPropeller *SimulationMaster::propeller() {
    return m_propeller;
}

void SimulationMaster::serialize(QDataStream &stream) {
    foreach(auto d, getDevices())
        d->serialize(stream);
    stream << int(EndOfStreamMagic);
}

void SimulationMaster::deserialize(QDataStream &stream) {
    foreach(auto d, getDevices())
        d->deserialize(stream);
    int eom=0;
    stream >> eom;
    if (eom != int(EndOfStreamMagic))
        throw HWSerializeException("invalid end of simulation stream");
}

void SimulationMaster::compareWith(SimulationMaster *expected) {
    //vergleicht, das zwei simulatoren exakt gleich laufen
    auto *expectedProp = expected->m_propeller;
    const auto cnt = m_propeller->globalCounter();

    if (expectedProp->globalCounter() != cnt)
        qDebug()<<"Global Counter mismatch"<<cnt<<"expected:"<<expectedProp->globalCounter();
    if (expectedProp->pinOut() != m_propeller->pinOut())
        qDebug()<<"Pinout mismatch"<<m_propeller->globalCounter()<<QString::number(m_propeller->pinOut(),16).rightJustified(8)<<"expected:"<<QString::number(expectedProp->pinOut(),16).rightJustified(8);
    if (expectedProp->pinDir() != m_propeller->pinDir())
        qDebug()<<"Pindir mismatch"<<m_propeller->globalCounter()<<QString::number(m_propeller->pinDir(),16).rightJustified(8)<<"expected:"<<QString::number(expectedProp->pinDir(),16).rightJustified(8);
    for (int cog=0; cog<8; ++cog) {
        auto expectedStatus = expectedProp->cogStatus(cog);
        auto thisStatus = m_propeller->cogStatus(cog);
        if (expectedStatus.running != thisStatus.running)
            qDebug()<<cnt<<"Cog Comp Mismatch: Cog"<<cog<<"running mismatch"<<thisStatus.running<<"expected:"<<expectedStatus.running;
        if (expectedStatus.enabled != thisStatus.enabled)
            qDebug()<<cnt<<"Cog Comp Mismatch: Cog"<<cog<<"enabled mismatch"<<thisStatus.enabled<<"expected:"<<expectedStatus.enabled;
        if (expectedStatus.programCounter != thisStatus.programCounter)
            qDebug()<<cnt<<"Cog Comp Mismatch: Cog"<<cog<<"programCounter mismatch"<<thisStatus.programCounter<<"expected:"<<expectedStatus.programCounter;
        if (expectedStatus.state != thisStatus.state)
            qDebug()<<cnt<<"Cog Comp Mismatch: Cog"<<cog<<"State mismatch"<<thisStatus.stateName()<<"expected:"<<expectedStatus.stateName();
        if (expectedStatus.zeroFlag != thisStatus.zeroFlag || expectedStatus.carryFlag != thisStatus.carryFlag)
            qWarning()<<cnt<<"Cog Flag mismatch"<<m_propeller->globalCounter()<<cog;
        if (expectedProp->cogCounterStatus(cog, 0) != m_propeller->cogCounterStatus(cog, 0)) {
            qWarning()<<"Cog Counter A mismatch"<<cnt<<cog;
            qWarning()<<"   expected:"<<expectedProp->cogCounterStatus(cog, 0).toString();
            qWarning()<<"   got:     "<<m_propeller->cogCounterStatus(cog, 0).toString();
        }
        if (expectedProp->cogCounterStatus(cog, 1) != m_propeller->cogCounterStatus(cog, 1)) {
            qWarning()<<"Cog Counter B mismatch"<<cnt<<cog;
            qWarning()<<"   expected:"<<expectedProp->cogCounterStatus(cog, 1).toString();
            qWarning()<<"   got:     "<<m_propeller->cogCounterStatus(cog, 1).toString();
        }
        for (int j=0; j<512; ++j) {
            if (expectedProp->cogMemory(cog, j) != m_propeller->cogMemory(cog, j))
                qWarning()<<"Cog Memory mismatch"<<cnt<<cog<<"at:"<<j<<"got:"<<m_propeller->cogMemory(cog,j)<<"expected:"<<expectedProp->cogMemory(cog,j);
        }
    }
    for (int i=0; i<16384; ++i) {
        if (expectedProp->hubMemory(i) != m_propeller->hubMemory(i))
            qWarning()<<"Hub Memory mismatch"<<cnt<<"at:"<<i<<"got:"<<m_propeller->hubMemory(i)<<"expected:"<<expectedProp->hubMemory(i);
    }
}

PS2Keyboard *SimulationMaster::keyboard() {
    return m_keyboard;
}
