#ifndef SIMULATIONMASTER_H
#define SIMULATIONMASTER_H

#include "abstractpropeller.h"
#include "i2cdevice.h"
#include "vgadecoder.h"
#include "ps2device.h"
#include "i2ceeprom.h"
#include <QFile>

class SimulationMaster : public QObject
{
    Q_OBJECT
public:
    enum PropellerImplementation {PropGenCpp,PropNewCpp,PropVerilator,PropAsm};
    SimulationMaster(PropellerImplementation impl);
    void clockCycle();
    QList<HWTraceable*> getDevices();
    AbstractPropeller *propeller();
    PS2Keyboard *keyboard();

    void serialize(QDataStream &stream);
    void deserialize(QDataStream &stream);
    void compareWith(SimulationMaster *expected);
signals:
    void nextFrame(QImage frame);
private:
    static AbstractPropeller* propellerFactory(PropellerImplementation impl, QString deviceId, QString deviceName);
    enum {EndOfStreamMagic = 0xDEADCAFE};
    AbstractPropeller *m_propeller;
    I2CEndpoint *m_bootEEPromI2CEndpoint;
    I2CEEPRom *m_bootEEProm;
    VGADecoder *m_vgaDecoder;
    PS2Keyboard *m_keyboard;

    QFile m_vcdLogFile;
    quint32 m_lastVCDPins;
    quint32 m_lowSpeedSkipCounter;
    quint32 m_lastSDOPins;
};

#endif // SIMULATIONMASTER_H
